import os
import numpy as np

from ....definitions import MAIN_DIR
from ....Classes.MeshMat import MeshMat
from ....Classes.CellMat import CellMat
from ....Classes.PointMat import PointMat
from ....Classes.RefTriangle3 import RefTriangle3

from os.path import join

from ....Functions.FEMM import FEMM_GROUPS


def get_meshsolution(self, femm, save_path, j_t0, id_worker=0, is_get_mesh=False):
    """Load the mesh data and solution data. FEMM must be working and a simulation must have been solved.

    Parameters
    ----------
    self : MagFEMM
        a MagFEMM object
    femm : FEMMHandler
        client to send command to a FEMM instance
    save_path: str
        Full path to folder in which to save results
    j_t0 : int
        time step index
    id_worker : int
        worker index
    is_get_mesh : bool
        True to load and create the mesh or not

    Returns
    -------
    mesh: MeshMat
        Object containing magnetic mesh
    B: ndarray
        3D Magnetic flux density for each element (Nelem, 3) [T]
    H : ndarray
        3D Magnetic field for each element (Nelem, 3) [A/m]
    mu : ndarray
        Magnetic relative permeability for each element (Nelem,1) []
    groups: dict
        Dict whose values are group label and values are array of indices of related elements

    """

    idworker = str(id_worker)  # For parallelization TODO

    path_txt = join(MAIN_DIR, "Functions", "FEMM") + "\\"
    path_txt_lua = path_txt.replace("\\", "/")
    path_lua_in = join(path_txt, "get_mesh_data_FEMM.lua")
    path_lua_out = join(save_path, "get_mesh_data_FEMM" + idworker + ".lua")
    path_txt_out = save_path + "\\"
    path_txt_out = path_txt_out.replace("\\", "/")

    # Create a new LUA script with current paths
    file_lua = open(path_lua_in, "r")
    text_lua = file_lua.read()
    file_lua.close()
    text_lua = text_lua.replace("my_path_txt", path_txt_out)
    text_lua = text_lua.replace("my_id_worker", idworker)

    file_lua_out = open(path_lua_out, "w")
    file_lua_out.write(text_lua)
    file_lua_out.close()

    # Run the LUA externally using FEMM LUA console and store the data in the
    # temporary text files
    path_lua_out2 = path_lua_out.replace("\\", "/")
    femm.callfemm('dofile("' + path_lua_out2 + '")')

    # Delete the LUA script
    os.remove(path_lua_out)

    # Read the nodes and elements files
    path_node = join(save_path, "nodes" + idworker + ".txt")
    path_element = join(save_path, "elements" + idworker + ".txt")
    listNd0 = np.loadtxt(path_node, delimiter=" ")
    listElem0 = np.loadtxt(path_element, dtype="i", delimiter=" ")
    NbNd = len(listNd0)
    NbElem = len(listElem0)

    # Node list
    listNd = np.zeros(shape=(NbNd, 3))
    listNd[:, 0] = listNd0[:, 0]
    listNd[:, 1] = listNd0[:, 1]

    # Element list
    # listElem = np.zeros(shape=(NbElem, 3))
    listElem = listElem0[:, 0:3] - 1

    # Delete text files
    os.remove(path_node)
    os.remove(path_element)

    # Read the results file
    path_results = join(save_path, "results" + idworker + ".txt")
    results = np.loadtxt(path_results, delimiter=" ")

    # Delete text files
    os.remove(path_results)

    ## Create Mesh and Solution dictionaries

    # Save MeshMat for only 1 time step with sliding band
    if is_get_mesh:
        mesh = MeshMat()
        mesh.label = "FEMM"
        mesh.cell["triangle"] = CellMat(
            connectivity=listElem,
            nb_cell=NbElem,
            nb_pt_per_cell=3,
            indice=np.linspace(0, NbElem - 1, NbElem, dtype=int),
        )
        mesh.cell["triangle"].interpolation.ref_cell = RefTriangle3(epsilon=1e-9)
        mesh.point = PointMat(
            coordinate=listNd[:, 0:2], nb_pt=NbNd, indice=np.linspace(0, NbNd - 1, NbNd)
        )
        # get all groups that are in the FEMM model
        groups = dict()
        for grp in FEMM_GROUPS:
            idx = FEMM_GROUPS[grp]["ID"]
            name = FEMM_GROUPS[grp]["name"]
            ind = np.where(listElem0[:, 6] == idx)[0]
            if ind.size > 0:
                groups[name] = mesh.cell["triangle"].indice[ind]
    else:
        mesh = None
        groups = None

    B = results[:, 0:2]
    H = results[:, 2:4]
    mu = results[:, 4]

    return mesh, B, H, mu, groups



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Mesh/Interpolation/RefCell.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Mesh/RefCell
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Mesh.RefCell.interpolation import interpolation
except ImportError as error:
    interpolation = error


from ._check import InitUnKnowClassError


class RefCell(FrozenClass):
    """Store shape functions definition in the reference element"""

    VERSION = 1

    # cf Methods.Mesh.RefCell.interpolation
    if isinstance(interpolation, ImportError):
        interpolation = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use RefCell method interpolation: " + str(interpolation)
                )
            )
        )
    else:
        interpolation = interpolation
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, epsilon=0.05, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "epsilon" in list(init_dict.keys()):
                epsilon = init_dict["epsilon"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.epsilon = epsilon

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        RefCell_str = ""
        if self.parent is None:
            RefCell_str += "parent = None " + linesep
        else:
            RefCell_str += "parent = " + str(type(self.parent)) + " object" + linesep
        RefCell_str += "epsilon = " + str(self.epsilon) + linesep
        return RefCell_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.epsilon != self.epsilon:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._epsilon != self._epsilon:
            diff_list.append(name + ".epsilon")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.epsilon)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        RefCell_dict = dict()
        RefCell_dict["epsilon"] = self.epsilon
        # The class name is added to the dict for deserialisation purpose
        RefCell_dict["__class__"] = "RefCell"
        return RefCell_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.epsilon = None

    def _get_epsilon(self):
        """getter of epsilon"""
        return self._epsilon

    def _set_epsilon(self, value):
        """setter of epsilon"""
        check_var("epsilon", value, "float", Vmin=0.00e00)
        self._epsilon = value

    epsilon = property(
        fget=_get_epsilon,
        fset=_set_epsilon,
        doc=u"""Precision criterion

        :Type: float
        :min: 0.00E+00
        """,
    )



# -*- coding: utf-8 -*-
from ....Methods.Simulation.Input import InputError


def run(self):
    """Run the Magnetics module"""
    if self.parent is None:
        raise InputError(
            "ERROR: The Magnetic object must be in a Simulation object to run"
        )
    if self.parent.parent is None:
        raise InputError(
            "ERROR: The Simulation object must be in an Output object to run"
        )

    self.get_logger().info("Starting Magnetic module")
    output = self.parent.parent

    # Compute and store time and angle axes from elec output
    # and returns additional axes in axes_dict
    axes_dict = self.comp_axes(output)

    # Calculate airgap flux
    out_dict = self.comp_flux_airgap(output, axes_dict)

    # Store magnetic quantities contained in out_dict in OutMag, as Data object if necessary
    output.mag.store(out_dict, axes_dict)



from ....Methods.Geometry.Arc3 import PointTranslateArc3Error


def translate(self, Zt):
    """Translate the Arc3 object with

    Parameters
    ----------
    self : Arc3
        An Arc3 object

    Zt : complex
        Complex value for translation


    Returns
    -------
    None

    """
    if (
        not isinstance(Zt, complex)
        and not isinstance(Zt, int)
        and not isinstance(Zt, float)
    ):
        raise PointTranslateArc3Error(
            "The point must be a complex number or int or float"
        )

    # check if the Arc3 is correct
    self.check()

    # Modification from the translation of Arc3
    self.begin = self.begin + Zt
    self.end = self.end + Zt



# -*- coding: utf-8 -*-


def comp_height_yoke(self):
    """Compute the yoke height

    Parameters
    ----------
    self : LamHole
        A LamHole object

    Returns
    -------
    Hy: float
        yoke height [m]

    """

    if self.is_internal:
        R = self.Rext
    else:
        R = self.Rint

    # The yoke is define the greater cylinder without holes
    for hole in self.hole:
        (Rmin, Rmax) = hole.comp_radius()
        if self.is_internal:
            R = min(R, Rmin)
        else:
            R = max(R, Rmax)

    if self.is_internal:
        return R - self.Rint
    else:
        return self.Rext - R



# -*- coding: utf-8 -*-


def get_end(self):
    """Return the end point of the arc

    Parameters
    ----------
    self : Arc3
        An Arc3 object

    Returns
    -------
    end: complex
        Complex coordinates of the end point of the Arc3
    """

    return self.end



# -*- coding: utf-8 -*-

from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve needed to plot the object.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotW24
        A SlotW24 object

    Returns
    -------
    curve_list: list
        A list of 2 Segment and 1 Arc

    """

    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Zc = 0

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Arc1(Z2, Z3, abs(Z2)))
    curve_list.append(Segment(Z3, Z4))

    return curve_list



# -*- coding: utf-8 -*-
from numpy import exp, pi
import cmath

from ...Classes.Arc import Arc
from ...Classes.Arc1 import Arc1
from ...Classes.Arc2 import Arc2
from ...Classes.Circle import Circle
from ...Classes.Segment import Segment
from ...Classes.SurfLine import SurfLine
from ...Classes.MachineSIPMSM import MachineSIPMSM


def get_sliding_band(sym, machine):
    """Returns  a list of surface in the airgap including the sliding band surface

    Parameters
    ----------
    sym: int
        Symmetry factor (1= full machine, 2= half of the machine...)
    Rgap_mec_int: float
        Internal lamination mechanic radius
    Rgap_mec_ext: float
        External lamination mechanic radius

    Returns
    -------
    surf_list: list
        List of surface in the airgap including the sliding band surface
    """
    lam_list = machine.get_lam_list()
    lam_int = lam_list[0]
    lam_ext = lam_list[1]

    stator_list = machine.stator.build_geometry(sym=sym)
    rotor_list = machine.rotor.build_geometry(sym=sym)
    slot_height = machine.stator.slot.comp_height()
    wind_slot_height = machine.stator.slot.comp_height_active()
    wedge_height = slot_height - wind_slot_height
    Rgap_mec_int = lam_int.comp_radius_mec()
    Rgap_mec_ext = lam_ext.comp_radius_mec()
    Rgap_mag_int = lam_int.Rext
    Rgap_mag_ext = lam_ext.Rint
    Sor = lam_ext.Rext
    Wgap_mec = Rgap_mec_ext - Rgap_mec_int
    W_sb = Wgap_mec / 4  # Width sliding band
    tol = 0.1e-3  # Tolerance
    max_radius_Airgap_ext = Rgap_mec_ext + wedge_height + tol
    min_radius_Airgap_int = Rgap_mec_int - tol

    # Find lines that are between Stator Inner Diam and slot wedge area
    stator_airgap_ext_lines = list()
    first_points = list()  # To check the loop and find inverted lines
    for surf in stator_list:
        for line in surf.get_lines():
            p1 = line.get_begin()
            p2 = line.get_end()
            if abs(p1) < max_radius_Airgap_ext and abs(p2) < max_radius_Airgap_ext:
                r_p1, phi_p1 = cmath.polar(p1)
                r_p2, phi_p2 = cmath.polar(p2)
                if isinstance(line, Arc):
                    p3 = line.get_center()
                    r_p3, phi_p3 = cmath.polar(p3)
                    radius = r_p1 - r_p3
                    line_copy = Arc1(
                        begin=p1,
                        end=p2,
                        radius=radius,
                        label=line.label,
                        is_trigo_direction=True,
                    )
                    # if phi_p2 > phi_p1:
                    #    line_copy.reverse()
                else:
                    line_copy = Segment(begin=p1, end=p2, label=line.label)
                new_p1 = p1
                for p in first_points:
                    if abs(p1.real - p.real) < tol and abs(p1.imag - p.imag) < tol:
                        line_copy.reverse()
                        new_p1 = p2
                        break
                stator_airgap_ext_lines.append(line_copy)
                first_points.append(new_p1)

    # Find lines that are between Rotor Outer Diam and ...
    rotor_airgap_int_lines = list()
    if isinstance(machine, MachineSIPMSM):
        min_radius_Airgap_int = min(Rgap_mec_int, Rgap_mag_int) - tol
        for surf in rotor_list:
            if "Magnet" in surf.label:
                continue
            for line in surf.get_lines():
                if line.comp_length() < tol:
                    continue
                p1 = line.get_begin()
                p2 = line.get_end()

                if abs(p1) > min_radius_Airgap_int and abs(p2) > min_radius_Airgap_int:
                    r_p1, phi_p1 = cmath.polar(p1)
                    r_p2, phi_p2 = cmath.polar(p2)
                    if isinstance(line, Arc):
                        p3 = line.get_center()
                        r_p3, phi_p3 = cmath.polar(p3)
                        radius = r_p1 - r_p3
                        line_copy = Arc1(
                            begin=p1,
                            end=p2,
                            radius=radius,
                            label=line.label,
                            is_trigo_direction=True,
                        )
                        if phi_p2 > phi_p1:
                            line_copy.reverse()
                    else:
                        line_copy = Segment(begin=p1, end=p2, label=line.label)
                        if phi_p2 > phi_p1:
                            line_copy.reverse()
                    rotor_airgap_int_lines.append(line_copy)
    else:
        for surf in rotor_list:
            for line in surf.get_lines():
                p1 = line.get_begin()
                p2 = line.get_end()
                if abs(p1) > min_radius_Airgap_int and abs(p2) > min_radius_Airgap_int:
                    r_p1, phi_p1 = cmath.polar(p1)
                    r_p2, phi_p2 = cmath.polar(p2)
                    if isinstance(line, Arc):
                        p3 = line.get_center()
                        r_p3, phi_p3 = cmath.polar(p3)
                        radius = r_p1 - r_p3
                        line_copy = Arc1(
                            begin=p1,
                            end=p2,
                            radius=radius,
                            label=line.label,
                            is_trigo_direction=True,
                        )
                        if phi_p2 > phi_p1:
                            line_copy.reverse()
                    else:
                        line_copy = Segment(begin=p1, end=p2, label=line.label)
                        if phi_p2 > phi_p1:
                            line_copy.reverse()
                    rotor_airgap_int_lines.append(line_copy)

    surf_list = list()
    if sym == 1:  # Complete machine
        # TODO: Sliding Band Full Machine no implemented yet.
        # Internal AirGap
        surf_list.append(
            Circle(
                center=0,
                radius=Rgap_mec_int + W_sb,
                label="Airgap_int",
                point_ref=(Rgap_mec_int + W_sb / 2) * exp(1j * pi / 2),
                line_label="int_airgap_line",
            )
        )
        # Internal Sliding band
        surf_list.append(
            Circle(
                center=0,
                radius=Rgap_mec_int + 2 * W_sb,
                label="SildingBand_int",
                point_ref=(Rgap_mec_ext - W_sb / 2) * exp(1j * pi / 2),
                line_label="int_sb_line",
            )
        )
        # External AirGap
        surf_list.append(
            Circle(
                center=0,
                radius=Rgap_mec_int + 3 * W_sb,
                label="Airgap_ext",
                point_ref=(Rgap_mec_ext - W_sb / 2) * exp(1j * pi / 2),
                line_label="ext_airgap_line",
            )
        )

    else:  # Symmetry
        # Internal AirGap
        Z1 = Rgap_mec_int
        if isinstance(machine, MachineSIPMSM):
            Z1 = Rgap_mag_int
        Z0 = Z1 * exp(1j * 2 * pi / sym)
        Z2 = Rgap_mec_int + W_sb
        Z3 = Z2 * exp(1j * 2 * pi / sym)
        airgap_lines = list()
        airgap_lines.append(Segment(begin=Z1, end=Z2, label="int_airgap_line_1"))
        int_airgap_arc = Arc1(
            begin=Z2,
            end=Z3,
            radius=Rgap_mec_int + W_sb,
            label="int_airgap_arc",
            is_trigo_direction=True,
        )
        airgap_lines.append(int_airgap_arc)
        airgap_lines.append(Segment(begin=Z3, end=Z0, label="int_airgap_line_2"))
        airgap_lines.extend(rotor_airgap_int_lines)
        surf_list.append(
            SurfLine(
                line_list=airgap_lines,
                point_ref=(Z2 - W_sb / 2) * exp(1j * pi / sym),
                label="Airgap_int",
            )
        )

        # Internal Sliding Band
        Z4 = Rgap_mec_int + 2 * W_sb - tol / 10
        Z5 = Z4 * exp(1j * 2 * pi / sym)
        airgap_lines = list()
        airgap_lines.append(Segment(begin=Z2, end=Z4, label="int_sb_line_1"))
        int_sb_arc = Arc1(
            begin=Z4,
            end=Z5,
            radius=Rgap_mec_int + 2 * W_sb,
            label="int_sb_arc",
            is_trigo_direction=True,
        )
        airgap_lines.append(int_sb_arc)
        airgap_lines.append(Segment(begin=Z5, end=Z3, label="int_sb_line_2"))
        int_airgap_arc_copy = Arc1(
            begin=Z2,
            end=Z3,
            radius=Rgap_mec_int + W_sb,
            label="int_airgap_arc_copy",
            is_trigo_direction=True,
        )
        int_airgap_arc_copy.reverse()
        airgap_lines.append(int_airgap_arc_copy)
        surf_list.append(
            SurfLine(
                line_list=airgap_lines,
                point_ref=(Z4 - W_sb / 2) * exp(1j * pi / sym),
                label="SlidingBand_int",
            )
        )

        # External Sliding Band
        Z6 = Rgap_mec_int + 2 * W_sb + tol / 10
        Z7 = Z6 * exp(1j * 2 * pi / sym)
        Z8 = Rgap_mec_int + 3 * W_sb
        Z9 = Z8 * exp(1j * 2 * pi / sym)
        airgap_lines = list()
        airgap_lines.append(Segment(begin=Z6, end=Z8, label="ext_sb_line_1"))
        ext_airgap_arc = Arc1(
            begin=Z8,
            end=Z9,
            radius=Rgap_mec_int + 3 * W_sb,
            label="ext_airgap_arc",
            is_trigo_direction=True,
        )
        airgap_lines.append(ext_airgap_arc)
        airgap_lines.append(Segment(begin=Z9, end=Z7, label="ext_sb_line_2"))
        ext_sb_arc = Arc1(
            begin=Z6,
            end=Z7,
            radius=Rgap_mec_int + 2 * W_sb,
            label="ext_sb_arc",
            is_trigo_direction=True,
        )
        ext_sb_arc.reverse()
        airgap_lines.append(ext_sb_arc)
        surf_list.append(
            SurfLine(
                line_list=airgap_lines,
                point_ref=(Z8 - W_sb / 2) * exp(1j * pi / sym),
                label="SlidingBand_ext",
            )
        )

        # External AirGap
        Z10 = Rgap_mec_ext
        Z11 = Z10 * exp(1j * 2 * pi / sym)
        airgap_lines = list()
        airgap_lines.append(Segment(begin=Z8, end=Z10, label="ext_airgap_line_1"))
        airgap_lines.append(Segment(begin=Z11, end=Z9, label="ext_airgap_line_2"))
        ext_airgap_arc_copy = Arc1(
            begin=Z8,
            end=Z9,
            radius=Rgap_mec_int + 3 * W_sb,
            label="ext_airgap_arc_copy",
            is_trigo_direction=True,
        )
        ext_airgap_arc_copy.reverse()
        airgap_lines.append(ext_airgap_arc_copy)
        airgap_lines.extend(stator_airgap_ext_lines)
        surf_list.append(
            SurfLine(
                line_list=airgap_lines,
                point_ref=(Z8 + W_sb / 2) * exp(1j * pi / sym),
                label="Airgap_ext",
            )
        )

    return surf_list



# -*- coding: utf-8 -*-

import numpy as np


def scalar_product(self, funca, funcb, detJ, weights, nb_gauss_points):
    """Scalar product of shape functions with L2 gauss integration

    Parameters
    ----------
    self : ScalarProductL2
        a ScalarProductL2 object
    funca : ndarray
        vertice of the cell
    nba : ndarray
        coordinates of a point
    funcb : ndarray
        vertice of the cell
    nbb : ndarray
        coordinates of a point
    detJ : ndarray
        jacobian determinant evaluated for each gauss point
    weights : ndarray
        gauss weights
    nb_gauss_points : int
        number of gauss points

    Returns
    -------
    l2_scal : ndarray
        a L2 scalar product
    """

    func_a_w_dJ = np.zeros(funca.shape)
    for i in range(nb_gauss_points):
        func_a_w_dJ[i, :] = funca[i, :] * weights[i] * detJ[i]

    l2_scal_mat = np.squeeze(np.tensordot(func_a_w_dJ, funcb, axes=([0], [0])))

    return l2_scal_mat



from ....Methods.Import import ImportError


def check(self):
    """Check that the object is correctly set"""
    if self.start is None:
        raise ImportError("Start must not be None")
    if self.stop is None:
        raise ImportError("Stop must not be None")
    if self.num is None:
        raise ImportError("Num must not be None")
    if self.stop <= self.start:
        raise ImportError("You must have start < stop")
    if self.endpoint is None:
        self.endpoint = True



# -*- coding: utf-8 -*-


def comp_length(self):
    """Compute the length of the line

    Parameters
    ----------
    self : Segment
        A Segment object

    Returns
    -------
    length: float
        lenght of the line [m]

    Raises
    ------
    PointSegmentError
        Call Segment.check()

    """

    self.check()

    z1 = self.begin
    z2 = self.end

    return float(abs(z2 - z1))



# -*- coding: utf-8 -*-

# File generated according to SMHoleMag.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Tools.HelpButton import HelpButton

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SMHoleMag(object):
    def setupUi(self, SMHoleMag):
        if not SMHoleMag.objectName():
            SMHoleMag.setObjectName(u"SMHoleMag")
        SMHoleMag.resize(780, 640)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SMHoleMag.sizePolicy().hasHeightForWidth())
        SMHoleMag.setSizePolicy(sizePolicy)
        SMHoleMag.setMinimumSize(QSize(780, 640))
        self.verticalLayout = QVBoxLayout(SMHoleMag)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.b_help = HelpButton(SMHoleMag)
        self.b_help.setObjectName(u"b_help")
        self.b_help.setPixmap(QPixmap(u":/images/images/icon/help_16.png"))

        self.horizontalLayout_2.addWidget(self.b_help)

        self.out_hole_pitch = QLabel(SMHoleMag)
        self.out_hole_pitch.setObjectName(u"out_hole_pitch")

        self.horizontalLayout_2.addWidget(self.out_hole_pitch)

        self.horizontalSpacer_3 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer_3)

        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.b_add = QPushButton(SMHoleMag)
        self.b_add.setObjectName(u"b_add")

        self.horizontalLayout_3.addWidget(self.b_add)

        self.b_remove = QPushButton(SMHoleMag)
        self.b_remove.setObjectName(u"b_remove")

        self.horizontalLayout_3.addWidget(self.b_remove)

        self.horizontalSpacer_2 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_3.addItem(self.horizontalSpacer_2)

        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Preferred
        )

        self.verticalLayout.addItem(self.verticalSpacer)

        self.tab_hole = QTabWidget(SMHoleMag)
        self.tab_hole.setObjectName(u"tab_hole")
        self.tab_hole.setMinimumSize(QSize(770, 500))

        self.verticalLayout.addWidget(self.tab_hole)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.b_plot = QPushButton(SMHoleMag)
        self.b_plot.setObjectName(u"b_plot")

        self.horizontalLayout.addWidget(self.b_plot)

        self.b_previous = QPushButton(SMHoleMag)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout.addWidget(self.b_previous)

        self.b_next = QPushButton(SMHoleMag)
        self.b_next.setObjectName(u"b_next")
        self.b_next.setEnabled(True)

        self.horizontalLayout.addWidget(self.b_next)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(SMHoleMag)

        self.tab_hole.setCurrentIndex(-1)

        QMetaObject.connectSlotsByName(SMHoleMag)

    # setupUi

    def retranslateUi(self, SMHoleMag):
        SMHoleMag.setWindowTitle(QCoreApplication.translate("SMHoleMag", u"Form", None))
        self.b_help.setText("")
        self.out_hole_pitch.setText(
            QCoreApplication.translate("SMHoleMag", u"Slot pitch = 2*Pi / Zs = ", None)
        )
        self.b_add.setText(
            QCoreApplication.translate("SMHoleMag", u"Add new slots", None)
        )
        self.b_remove.setText(
            QCoreApplication.translate("SMHoleMag", u"Remove last slot", None)
        )
        self.b_plot.setText(QCoreApplication.translate("SMHoleMag", u"Preview", None))
        self.b_previous.setText(
            QCoreApplication.translate("SMHoleMag", u"Previous", None)
        )
        self.b_next.setText(QCoreApplication.translate("SMHoleMag", u"Save", None))

    # retranslateUi



# -*- coding: utf-8 -*-
from numpy import arcsin, sin


def comp_surface_active(self):
    """Compute the Slot active inner surface (by analytical computation)

    Parameters
    ----------
    self : SlotM13
        A SlotM13 object

    Returns
    -------
    Swind: float
        Slot active inner surface [m**2]

    """

    point_dict = self._comp_point_coordinate()
    ZM1 = point_dict["ZM1"]
    ZM2 = point_dict["ZM2"]
    alpha = 2 * float(arcsin(self.Wmag / (2 * self.Rtopm)))
    Sarc = (self.Rtopm ** 2.0) / 2.0 * (alpha - sin(alpha))

    S1 = abs(ZM1 - ZM2) * self.Wmag
    return S1 + Sarc



# -*- coding: utf-8 -*-

from numpy import arcsin, cos, exp, pi, sin, tan


def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW14
        A SlotW14 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """

    Rbo = self.get_Rbo()

    point_dict = self._comp_point_coordinate()
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]

    if self.is_outwards():
        return abs(Z5) - Rbo
    else:
        return Rbo - abs(Z4)



# -*- coding: utf-8 -*-
from numpy import arcsin, pi


def comp_angle_opening(self):
    """Compute the average opening angle of the Slot

    Parameters
    ----------
    self : SlotW24
        A SlotW24 object

    Returns
    -------
    alpha: float
        Average opening angle of the slot [rad]

    """

    alpha = float(arcsin(self.W3 / (2 * self.get_Rbo())))
    return 2 * pi / self.Zs - 2 * alpha



# -*- coding: utf-8 -*-

from numpy import real, min as np_min, max as np_max

from ....Classes.MeshMat import MeshMat
from ....Classes.MeshVTK import MeshVTK
from ....definitions import config_dict

COLOR_MAP = config_dict["PLOT"]["COLOR_DICT"]["COLOR_MAP"]


def plot_deflection(
    self,
    label=None,
    index=None,
    indices=None,
    clim=None,
    factor=None,
    field_name=None,
    ifreq=0,
    save_path=None,
):
    """Plot the operational deflection shape using pyvista plotter.

    Parameters
    ----------
    self : MeshSolution
        a MeshSolution object
    label : str
        a label
    index : int
        an index
    indices : list
        list of the points to extract (optional)
    clim : list
        a list of 2 elements for the limits of the colorbar
    factor : float
        factor to multiply vector field
    field_name : str
        title of the field to display on plot
    ifreq : int
        index of the frequency to use for plot (if exists)

    Returns
    -------
    """

    if save_path is None:
        try:
            import pyvistaqt as pv

            is_pyvistaqt = True
        except:
            import pyvista as pv

            is_pyvistaqt = False
    else:
        import pyvista as pv

        is_pyvistaqt = False

    if save_path is None:
        try:
            import pyvistaqt as pv

            is_pyvistaqt = True
        except:
            import pyvista as pv

            is_pyvistaqt = False
    else:
        import pyvista as pv

        is_pyvistaqt = False

    # Get the mesh
    mesh = self.get_mesh(label=label, index=index)
    if isinstance(mesh, MeshMat):
        mesh_pv = mesh.get_mesh_pv(indices=indices)
        mesh = MeshVTK(mesh=mesh_pv, is_pyvista_mesh=True)

    # Get the field
    field = real(
        self.get_field(
            label=label, index=index, indices=indices, is_surf=True, is_radial=True
        )
    )
    vect_field = real(self.get_field(label=label, index=index, indices=indices))
    if len(field.shape) == 2:
        # Third dimension is frequencies
        field = field[:, ifreq]
        vect_field = vect_field[:, :, ifreq]
    if field_name is None:
        if label is not None:
            field_name = label
        elif self.get_solution(index=index).label is not None:
            field_name = self.get_solution(index=index).label
        else:
            field_name = "Field"

    # Compute colorbar boundaries
    if clim is None:
        clim = [np_min(real(field)), np_max(real(field))]
        if (clim[1] - clim[0]) / clim[1] < 0.01:
            clim[0] = -clim[1]

    # Compute deformation factor
    if factor is None:
        factor = 1 / (100 * clim[1])

    # Extract surface
    surf = mesh.get_surf(indices=indices)

    # Add field to surf
    surf.vectors = real(vect_field) * factor

    # Warp by vectors
    surf_warp = surf.warp_by_vector()

    # Add radial field
    surf_warp[field_name] = real(field)

    # Configure plot
    if is_pyvistaqt:
        p = pv.BackgroundPlotter()
        p.set_background("white")
    else:
        pv.set_plot_theme("document")
        p = pv.Plotter(notebook=False)
    sargs = dict(
        interactive=True,
        title_font_size=20,
        label_font_size=16,
        font_family="arial",
        color="black",
    )
    p.add_mesh(
        surf_warp,
        scalars=field_name,
        opacity=1,
        show_edges=False,
        cmap=COLOR_MAP,
        clim=clim,
        scalar_bar_args=sargs,
    )
    if self.dimension == 2:
        p.view_xy()
    if save_path is None:
        p.show()
    else:
        p.show(interactive=False, screenshot=save_path)



import matplotlib.pyplot as plt
import numpy as np

from ....definitions import config_dict
from ....Methods.Output.XOutput import _get_symbol_data_

COLORS = config_dict["PLOT"]["COLOR_DICT"]["CURVE_COLORS"]
FONT_NAME = config_dict["PLOT"]["FONT_NAME"]


class PlotError(Exception):
    pass


def plot_multi(
    self,
    x_symbol,
    y_symbol,
    c_symbol=None,
    cmap=None,
    plot_type="scatter",
    idx=None,
    ax=None,
    title=None,
    is_show_fig=True,
    save_path=None,
):
    """
    Plot data from a DataKeeper for a given parameter from a ParamExplorer

    x_symbol: str
        ParamExplorer or DataKeeper symbol
    y_symbol: str
        ParamExplorer or DataKeeper symbol
    c_symbol: str
        optional symbol to set the plot colors
    cmap: colormap
        optional colormap
    plot_type: str
        scatter or plot to chose plot type
    idx: slice
        To plot only some data
    ax: matplotlib.pyplot.Axe
        To put the plot in a specific ax
    title: str
        Figure or subfigure title according to ax
    is_show_fig : bool
        True to show figure after plot
    save_path : str
        full path of the png file where the figure is saved if save_path is not None

    Returns
    -------
    fig or ax

    Raises
    ------
    PlotError
    """

    if idx is None:
        idx = slice(None)

    if plot_type not in ["scatter", "plot"]:
        raise PlotError("Unknown plot_type {}.".format(plot_type))

    # get data and labels
    x_values, x_label = _get_symbol_data_(self, x_symbol, idx)
    y_values, y_label = _get_symbol_data_(self, y_symbol, idx)

    if c_symbol is None:
        colors = COLORS[0]
    else:
        # get the color data
        c_values, _ = _get_symbol_data_(self, c_symbol, idx)
        colors = c_values[:, np.newaxis]

    if cmap is None:
        cmap = plt.cm.jet

    # create new figure or use existing axis
    RET_FIG = False
    if ax is None:
        fig, ax = plt.subplots()
        fig.suptitle(title, fontname=FONT_NAME)
        RET_FIG = True
    else:
        ax.set_title(title, fontname=FONT_NAME)

    # plot
    if plot_type == "scatter":
        plot = ax.scatter(x_values, y_values, c=colors)
    elif plot_type == "plot":
        sort_index = np.argsort(x_values)
        plot = ax.plot(x_values[sort_index], y_values[sort_index], c=colors)

    # add legend
    if c_symbol is not None:
        legend1 = ax.legend(*plot.legend_elements(), loc="upper right", title=c_symbol)
        ax.add_artist(legend1)

    ax.set_xlabel(x_label, fontname=FONT_NAME)
    ax.set_ylabel(y_label, fontname=FONT_NAME)

    if save_path is not None:
        fig.savefig(save_path)
        plt.close()

    if is_show_fig:
        fig.show()

    # return
    if RET_FIG:
        return fig
    else:
        return ax



# -*- coding: utf-8 -*-

from numpy import pi
from ....Classes.LamSlot import LamSlot


def comp_volumes(self):
    """Compute the Lamination volumes (Vlam, Vvent, Vslot, Vmag)

    Parameters
    ----------
    self : LamSlotMag
        A LamSlotMag object

    Returns
    -------
    V_dict: dict
        Lamination surface dictionnary (Vlam, Vvent, Vslot, Vmag) [m**3]

    """

    V_dict = LamSlot.comp_volumes(self)
    Vmag = self.slot.comp_surface_active() * self.magnet.Lmag

    V_dict["Vmag"] = Vmag

    return V_dict



# -*- coding: utf-8 -*-

from numpy import angle, linspace
from scipy.optimize import fsolve

from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine
import matplotlib.pyplot as plt


def build_geometry_active(self, Nrad, Ntan, is_simplified=False, alpha=0, delta=0):
    """Split the slot active area in several zone
    This method assume that the active area is centered on X axis and symetrical
    Otherwise a dedicated build_geometry_active method must be provided

    Parameters
    ----------
    self : Slot
        A Slot object
    Nrad : int
        Number of radial layer
    Ntan : int
        Number of tangentiel layer
    is_simplified : bool
        boolean to specify if coincident lines are considered as one or different lines (Default value = False)
    alpha : float
        Angle for rotation (Default value = 0) [rad]
    delta : Complex
        complex for translation (Default value = 0)

    Returns
    -------
    surf_list:
        List of surface delimiting the active zone

    """

    assert Ntan in [1, 2]

    surf_act = self.get_surface_active()

    # Find the two intersection point with Ox axis
    inter_list = list()
    for line in surf_act.get_lines():
        inter_list.extend(line.intersect_line(0, 100))
    # When the two lines at the bottom cross on X axis (ex SlotW14)
    if len(inter_list) == 3 and abs(inter_list[0] - inter_list[1]) < 1e-6:
        inter_list.pop(0)
    assert len(inter_list) == 2

    if abs(inter_list[0]) < abs(inter_list[1]) and self.is_outwards():
        Ztan1 = inter_list[0]
        Ztan2 = inter_list[1]
    elif abs(inter_list[0]) > abs(inter_list[1]) and self.is_outwards():
        Ztan1 = inter_list[1]
        Ztan2 = inter_list[0]
    elif abs(inter_list[0]) < abs(inter_list[1]) and not self.is_outwards():
        Ztan1 = inter_list[1]
        Ztan2 = inter_list[0]
    elif abs(inter_list[0]) > abs(inter_list[1]) and not self.is_outwards():
        Ztan1 = inter_list[0]
        Ztan2 = inter_list[1]

    # First Tan split
    tan_list = list()
    if Ntan == 2:
        tan_list.append(
            surf_act.split_line(0, 100, is_top=False, is_join=True, label_join="")
        )
        tan_list.append(
            surf_act.split_line(0, 100, is_top=True, is_join=True, label_join="")
        )
    else:
        tan_list = [surf_act]

    # Rad split
    surf_list = list()
    X_list = linspace(Ztan1, Ztan2, Nrad + 1, True).tolist()[1:-1]
    for ii in range(Ntan):
        surf = tan_list[ii]
        if Nrad > 1:
            direct = self.is_outwards()
            for jj in range(Nrad - 1):
                X = X_list[jj]
                surf_list.append(
                    surf.split_line(
                        X - 100j, X + 100j, is_top=direct, is_join=True, label_join=""
                    )
                )
                surf = surf.split_line(
                    X - 100j, X + 100j, is_top=not direct, is_join=True, label_join=""
                )
            # Add the last surface
            surf_list.append(surf)
        else:  # add the radial surfaces without any other cut
            surf_list.append(surf.copy())

    # Set all label
    set_label(surf_list, Nrad, Ntan, self.get_name_lam())

    # Apply transformation
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list


def set_label(surf_list, Nrad, Ntan, st):
    """Set the normalized label"""

    index = 0
    for jj in range(Ntan):
        for ii in range(Nrad):
            surf_list[index].label = (
                "Wind_" + st + "_R" + str(ii) + "_T" + str(jj) + "_S0"
            )
            index += 1



# -*- coding: utf-8 -*-


def comp_height_active(self):
    """Compute the height of the winding area

    Parameters
    ----------
    self : SlotW23
        A SlotW23 object

    Returns
    -------
    Hwind: float
        Height of the winding area [m]

    """

    point_dict = self._comp_point_coordinate()
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z6 = point_dict["Z6"]

    if self.is_outwards():
        return abs(Z4) - abs((Z3 + Z6) / 2)
    else:
        return abs(Z3) - abs(Z4)



from math import ceil


def comp_mesh_dict(self, element_size, label="surface"):
    """Returns the number of mesh elements on each line of the surface
    to match the element_size.

    Parameters
    ----------
    self : Surface
        a Surface object

    element_size : float
        The size of each element on the mesh [m]
    label: str
        Label prefix for line without label


    Returns
    -------
    mesh_dict : dict
        Dictionary containing the number of element of each line of the surface
    """

    mesh_dict = dict()
    lines = self.get_lines()
    for ii, line in enumerate(lines):
        if line.label in [None, ""]:
            line.label = label + "_line_" + str(ii)

        length = line.comp_length()
        number_of_element = ceil(length / element_size)
        mesh_dict[line.label] = number_of_element

    return mesh_dict



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Output/OutElec.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Output/OutElec
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import set_array, check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Output.OutElec.get_Nr import get_Nr
except ImportError as error:
    get_Nr = error

try:
    from ..Methods.Output.OutElec.get_Is import get_Is
except ImportError as error:
    get_Is = error

try:
    from ..Methods.Output.OutElec.get_Us import get_Us
except ImportError as error:
    get_Us = error

try:
    from ..Methods.Output.OutElec.comp_I_mag import comp_I_mag
except ImportError as error:
    comp_I_mag = error


from numpy import array, array_equal
from ._check import InitUnKnowClassError
from .OutInternal import OutInternal


class OutElec(FrozenClass):
    """Gather the electric module outputs"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Output.OutElec.get_Nr
    if isinstance(get_Nr, ImportError):
        get_Nr = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutElec method get_Nr: " + str(get_Nr))
            )
        )
    else:
        get_Nr = get_Nr
    # cf Methods.Output.OutElec.get_Is
    if isinstance(get_Is, ImportError):
        get_Is = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutElec method get_Is: " + str(get_Is))
            )
        )
    else:
        get_Is = get_Is
    # cf Methods.Output.OutElec.get_Us
    if isinstance(get_Us, ImportError):
        get_Us = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutElec method get_Us: " + str(get_Us))
            )
        )
    else:
        get_Us = get_Us
    # cf Methods.Output.OutElec.comp_I_mag
    if isinstance(comp_I_mag, ImportError):
        comp_I_mag = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutElec method comp_I_mag: " + str(comp_I_mag))
            )
        )
    else:
        comp_I_mag = comp_I_mag
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Time=None,
        Angle=None,
        Is=None,
        Ir=None,
        angle_rotor=None,
        N0=None,
        angle_rotor_initial=0,
        logger_name="Pyleecan.Electrical",
        Tem_av_ref=None,
        Id_ref=None,
        Iq_ref=None,
        felec=None,
        Ud_ref=None,
        Uq_ref=None,
        Pj_losses=None,
        Pem_av_ref=None,
        Us=None,
        internal=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Time" in list(init_dict.keys()):
                Time = init_dict["Time"]
            if "Angle" in list(init_dict.keys()):
                Angle = init_dict["Angle"]
            if "Is" in list(init_dict.keys()):
                Is = init_dict["Is"]
            if "Ir" in list(init_dict.keys()):
                Ir = init_dict["Ir"]
            if "angle_rotor" in list(init_dict.keys()):
                angle_rotor = init_dict["angle_rotor"]
            if "N0" in list(init_dict.keys()):
                N0 = init_dict["N0"]
            if "angle_rotor_initial" in list(init_dict.keys()):
                angle_rotor_initial = init_dict["angle_rotor_initial"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "Tem_av_ref" in list(init_dict.keys()):
                Tem_av_ref = init_dict["Tem_av_ref"]
            if "Id_ref" in list(init_dict.keys()):
                Id_ref = init_dict["Id_ref"]
            if "Iq_ref" in list(init_dict.keys()):
                Iq_ref = init_dict["Iq_ref"]
            if "felec" in list(init_dict.keys()):
                felec = init_dict["felec"]
            if "Ud_ref" in list(init_dict.keys()):
                Ud_ref = init_dict["Ud_ref"]
            if "Uq_ref" in list(init_dict.keys()):
                Uq_ref = init_dict["Uq_ref"]
            if "Pj_losses" in list(init_dict.keys()):
                Pj_losses = init_dict["Pj_losses"]
            if "Pem_av_ref" in list(init_dict.keys()):
                Pem_av_ref = init_dict["Pem_av_ref"]
            if "Us" in list(init_dict.keys()):
                Us = init_dict["Us"]
            if "internal" in list(init_dict.keys()):
                internal = init_dict["internal"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.Time = Time
        self.Angle = Angle
        self.Is = Is
        self.Ir = Ir
        self.angle_rotor = angle_rotor
        self.N0 = N0
        self.angle_rotor_initial = angle_rotor_initial
        self.logger_name = logger_name
        self.Tem_av_ref = Tem_av_ref
        self.Id_ref = Id_ref
        self.Iq_ref = Iq_ref
        self.felec = felec
        self.Ud_ref = Ud_ref
        self.Uq_ref = Uq_ref
        self.Pj_losses = Pj_losses
        self.Pem_av_ref = Pem_av_ref
        self.Us = Us
        self.internal = internal

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OutElec_str = ""
        if self.parent is None:
            OutElec_str += "parent = None " + linesep
        else:
            OutElec_str += "parent = " + str(type(self.parent)) + " object" + linesep
        OutElec_str += "Time = " + str(self.Time) + linesep + linesep
        OutElec_str += "Angle = " + str(self.Angle) + linesep + linesep
        OutElec_str += "Is = " + str(self.Is) + linesep + linesep
        OutElec_str += "Ir = " + str(self.Ir) + linesep + linesep
        OutElec_str += (
            "angle_rotor = "
            + linesep
            + str(self.angle_rotor).replace(linesep, linesep + "\t")
            + linesep
            + linesep
        )
        OutElec_str += "N0 = " + str(self.N0) + linesep
        OutElec_str += (
            "angle_rotor_initial = " + str(self.angle_rotor_initial) + linesep
        )
        OutElec_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        OutElec_str += "Tem_av_ref = " + str(self.Tem_av_ref) + linesep
        OutElec_str += "Id_ref = " + str(self.Id_ref) + linesep
        OutElec_str += "Iq_ref = " + str(self.Iq_ref) + linesep
        OutElec_str += "felec = " + str(self.felec) + linesep
        OutElec_str += "Ud_ref = " + str(self.Ud_ref) + linesep
        OutElec_str += "Uq_ref = " + str(self.Uq_ref) + linesep
        OutElec_str += "Pj_losses = " + str(self.Pj_losses) + linesep
        OutElec_str += "Pem_av_ref = " + str(self.Pem_av_ref) + linesep
        OutElec_str += "Us = " + str(self.Us) + linesep + linesep
        if self.internal is not None:
            tmp = self.internal.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            OutElec_str += "internal = " + tmp
        else:
            OutElec_str += "internal = None" + linesep + linesep
        return OutElec_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.Time != self.Time:
            return False
        if other.Angle != self.Angle:
            return False
        if other.Is != self.Is:
            return False
        if other.Ir != self.Ir:
            return False
        if not array_equal(other.angle_rotor, self.angle_rotor):
            return False
        if other.N0 != self.N0:
            return False
        if other.angle_rotor_initial != self.angle_rotor_initial:
            return False
        if other.logger_name != self.logger_name:
            return False
        if other.Tem_av_ref != self.Tem_av_ref:
            return False
        if other.Id_ref != self.Id_ref:
            return False
        if other.Iq_ref != self.Iq_ref:
            return False
        if other.felec != self.felec:
            return False
        if other.Ud_ref != self.Ud_ref:
            return False
        if other.Uq_ref != self.Uq_ref:
            return False
        if other.Pj_losses != self.Pj_losses:
            return False
        if other.Pem_av_ref != self.Pem_av_ref:
            return False
        if other.Us != self.Us:
            return False
        if other.internal != self.internal:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.Time is None and self.Time is not None) or (
            other.Time is not None and self.Time is None
        ):
            diff_list.append(name + ".Time None mismatch")
        elif self.Time is not None:
            diff_list.extend(self.Time.compare(other.Time, name=name + ".Time"))
        if (other.Angle is None and self.Angle is not None) or (
            other.Angle is not None and self.Angle is None
        ):
            diff_list.append(name + ".Angle None mismatch")
        elif self.Angle is not None:
            diff_list.extend(self.Angle.compare(other.Angle, name=name + ".Angle"))
        if (other.Is is None and self.Is is not None) or (
            other.Is is not None and self.Is is None
        ):
            diff_list.append(name + ".Is None mismatch")
        elif self.Is is not None:
            diff_list.extend(self.Is.compare(other.Is, name=name + ".Is"))
        if (other.Ir is None and self.Ir is not None) or (
            other.Ir is not None and self.Ir is None
        ):
            diff_list.append(name + ".Ir None mismatch")
        elif self.Ir is not None:
            diff_list.extend(self.Ir.compare(other.Ir, name=name + ".Ir"))
        if not array_equal(other.angle_rotor, self.angle_rotor):
            diff_list.append(name + ".angle_rotor")
        if other._N0 != self._N0:
            diff_list.append(name + ".N0")
        if other._angle_rotor_initial != self._angle_rotor_initial:
            diff_list.append(name + ".angle_rotor_initial")
        if other._logger_name != self._logger_name:
            diff_list.append(name + ".logger_name")
        if other._Tem_av_ref != self._Tem_av_ref:
            diff_list.append(name + ".Tem_av_ref")
        if other._Id_ref != self._Id_ref:
            diff_list.append(name + ".Id_ref")
        if other._Iq_ref != self._Iq_ref:
            diff_list.append(name + ".Iq_ref")
        if other._felec != self._felec:
            diff_list.append(name + ".felec")
        if other._Ud_ref != self._Ud_ref:
            diff_list.append(name + ".Ud_ref")
        if other._Uq_ref != self._Uq_ref:
            diff_list.append(name + ".Uq_ref")
        if other._Pj_losses != self._Pj_losses:
            diff_list.append(name + ".Pj_losses")
        if other._Pem_av_ref != self._Pem_av_ref:
            diff_list.append(name + ".Pem_av_ref")
        if (other.Us is None and self.Us is not None) or (
            other.Us is not None and self.Us is None
        ):
            diff_list.append(name + ".Us None mismatch")
        elif self.Us is not None:
            diff_list.extend(self.Us.compare(other.Us, name=name + ".Us"))
        if (other.internal is None and self.internal is not None) or (
            other.internal is not None and self.internal is None
        ):
            diff_list.append(name + ".internal None mismatch")
        elif self.internal is not None:
            diff_list.extend(
                self.internal.compare(other.internal, name=name + ".internal")
            )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.Time)
        S += getsizeof(self.Angle)
        S += getsizeof(self.Is)
        S += getsizeof(self.Ir)
        S += getsizeof(self.angle_rotor)
        S += getsizeof(self.N0)
        S += getsizeof(self.angle_rotor_initial)
        S += getsizeof(self.logger_name)
        S += getsizeof(self.Tem_av_ref)
        S += getsizeof(self.Id_ref)
        S += getsizeof(self.Iq_ref)
        S += getsizeof(self.felec)
        S += getsizeof(self.Ud_ref)
        S += getsizeof(self.Uq_ref)
        S += getsizeof(self.Pj_losses)
        S += getsizeof(self.Pem_av_ref)
        S += getsizeof(self.Us)
        S += getsizeof(self.internal)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        OutElec_dict = dict()
        if self.Time is None:
            OutElec_dict["Time"] = None
        else:
            OutElec_dict["Time"] = self.Time.as_dict()
        if self.Angle is None:
            OutElec_dict["Angle"] = None
        else:
            OutElec_dict["Angle"] = self.Angle.as_dict()
        if self.Is is None:
            OutElec_dict["Is"] = None
        else:
            OutElec_dict["Is"] = self.Is.as_dict()
        if self.Ir is None:
            OutElec_dict["Ir"] = None
        else:
            OutElec_dict["Ir"] = self.Ir.as_dict()
        if self.angle_rotor is None:
            OutElec_dict["angle_rotor"] = None
        else:
            OutElec_dict["angle_rotor"] = self.angle_rotor.tolist()
        OutElec_dict["N0"] = self.N0
        OutElec_dict["angle_rotor_initial"] = self.angle_rotor_initial
        OutElec_dict["logger_name"] = self.logger_name
        OutElec_dict["Tem_av_ref"] = self.Tem_av_ref
        OutElec_dict["Id_ref"] = self.Id_ref
        OutElec_dict["Iq_ref"] = self.Iq_ref
        OutElec_dict["felec"] = self.felec
        OutElec_dict["Ud_ref"] = self.Ud_ref
        OutElec_dict["Uq_ref"] = self.Uq_ref
        OutElec_dict["Pj_losses"] = self.Pj_losses
        OutElec_dict["Pem_av_ref"] = self.Pem_av_ref
        if self.Us is None:
            OutElec_dict["Us"] = None
        else:
            OutElec_dict["Us"] = self.Us.as_dict()
        if self.internal is None:
            OutElec_dict["internal"] = None
        else:
            OutElec_dict["internal"] = self.internal.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        OutElec_dict["__class__"] = "OutElec"
        return OutElec_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Time = None
        self.Angle = None
        self.Is = None
        self.Ir = None
        self.angle_rotor = None
        self.N0 = None
        self.angle_rotor_initial = None
        self.logger_name = None
        self.Tem_av_ref = None
        self.Id_ref = None
        self.Iq_ref = None
        self.felec = None
        self.Ud_ref = None
        self.Uq_ref = None
        self.Pj_losses = None
        self.Pem_av_ref = None
        self.Us = None
        if self.internal is not None:
            self.internal._set_None()

    def _get_Time(self):
        """getter of Time"""
        return self._Time

    def _set_Time(self, value):
        """setter of Time"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Time"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Data()
        check_var("Time", value, "Data")
        self._Time = value

    Time = property(
        fget=_get_Time,
        fset=_set_Time,
        doc=u"""Electrical time Data object

        :Type: SciDataTool.Classes.DataND.Data
        """,
    )

    def _get_Angle(self):
        """getter of Angle"""
        return self._Angle

    def _set_Angle(self, value):
        """setter of Angle"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Angle"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Data()
        check_var("Angle", value, "Data")
        self._Angle = value

    Angle = property(
        fget=_get_Angle,
        fset=_set_Angle,
        doc=u"""Electrical position Data object

        :Type: SciDataTool.Classes.DataND.Data
        """,
    )

    def _get_Is(self):
        """getter of Is"""
        return self._Is

    def _set_Is(self, value):
        """setter of Is"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Is"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataND()
        check_var("Is", value, "DataND")
        self._Is = value

    Is = property(
        fget=_get_Is,
        fset=_set_Is,
        doc=u"""Stator currents DataTime object

        :Type: SciDataTool.Classes.DataND.DataND
        """,
    )

    def _get_Ir(self):
        """getter of Ir"""
        return self._Ir

    def _set_Ir(self, value):
        """setter of Ir"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Ir"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataND()
        check_var("Ir", value, "DataND")
        self._Ir = value

    Ir = property(
        fget=_get_Ir,
        fset=_set_Ir,
        doc=u"""Rotor currents as a function of time (each column correspond to one phase)

        :Type: SciDataTool.Classes.DataND.DataND
        """,
    )

    def _get_angle_rotor(self):
        """getter of angle_rotor"""
        return self._angle_rotor

    def _set_angle_rotor(self, value):
        """setter of angle_rotor"""
        if type(value) is int and value == -1:
            value = array([])
        elif type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("angle_rotor", value, "ndarray")
        self._angle_rotor = value

    angle_rotor = property(
        fget=_get_angle_rotor,
        fset=_set_angle_rotor,
        doc=u"""Rotor angular position as a function of time (if None computed according to Nr)

        :Type: ndarray
        """,
    )

    def _get_N0(self):
        """getter of N0"""
        return self._N0

    def _set_N0(self, value):
        """setter of N0"""
        check_var("N0", value, "float")
        self._N0 = value

    N0 = property(
        fget=_get_N0,
        fset=_set_N0,
        doc=u"""Rotor speed

        :Type: float
        """,
    )

    def _get_angle_rotor_initial(self):
        """getter of angle_rotor_initial"""
        return self._angle_rotor_initial

    def _set_angle_rotor_initial(self, value):
        """setter of angle_rotor_initial"""
        check_var("angle_rotor_initial", value, "float")
        self._angle_rotor_initial = value

    angle_rotor_initial = property(
        fget=_get_angle_rotor_initial,
        fset=_set_angle_rotor_initial,
        doc=u"""Initial angular position of the rotor at t=0

        :Type: float
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )

    def _get_Tem_av_ref(self):
        """getter of Tem_av_ref"""
        return self._Tem_av_ref

    def _set_Tem_av_ref(self, value):
        """setter of Tem_av_ref"""
        check_var("Tem_av_ref", value, "float")
        self._Tem_av_ref = value

    Tem_av_ref = property(
        fget=_get_Tem_av_ref,
        fset=_set_Tem_av_ref,
        doc=u"""Theorical Average Electromagnetic torque

        :Type: float
        """,
    )

    def _get_Id_ref(self):
        """getter of Id_ref"""
        return self._Id_ref

    def _set_Id_ref(self, value):
        """setter of Id_ref"""
        check_var("Id_ref", value, "float")
        self._Id_ref = value

    Id_ref = property(
        fget=_get_Id_ref,
        fset=_set_Id_ref,
        doc=u"""Electrical time vector (no symmetry)

        :Type: float
        """,
    )

    def _get_Iq_ref(self):
        """getter of Iq_ref"""
        return self._Iq_ref

    def _set_Iq_ref(self, value):
        """setter of Iq_ref"""
        check_var("Iq_ref", value, "float")
        self._Iq_ref = value

    Iq_ref = property(
        fget=_get_Iq_ref,
        fset=_set_Iq_ref,
        doc=u"""q-axis current magnitude

        :Type: float
        """,
    )

    def _get_felec(self):
        """getter of felec"""
        return self._felec

    def _set_felec(self, value):
        """setter of felec"""
        check_var("felec", value, "float")
        self._felec = value

    felec = property(
        fget=_get_felec,
        fset=_set_felec,
        doc=u"""Electrical Frequency

        :Type: float
        """,
    )

    def _get_Ud_ref(self):
        """getter of Ud_ref"""
        return self._Ud_ref

    def _set_Ud_ref(self, value):
        """setter of Ud_ref"""
        check_var("Ud_ref", value, "float")
        self._Ud_ref = value

    Ud_ref = property(
        fget=_get_Ud_ref,
        fset=_set_Ud_ref,
        doc=u"""d-axis voltage magnitude

        :Type: float
        """,
    )

    def _get_Uq_ref(self):
        """getter of Uq_ref"""
        return self._Uq_ref

    def _set_Uq_ref(self, value):
        """setter of Uq_ref"""
        check_var("Uq_ref", value, "float")
        self._Uq_ref = value

    Uq_ref = property(
        fget=_get_Uq_ref,
        fset=_set_Uq_ref,
        doc=u"""q-axis voltage magnitude

        :Type: float
        """,
    )

    def _get_Pj_losses(self):
        """getter of Pj_losses"""
        return self._Pj_losses

    def _set_Pj_losses(self, value):
        """setter of Pj_losses"""
        check_var("Pj_losses", value, "float")
        self._Pj_losses = value

    Pj_losses = property(
        fget=_get_Pj_losses,
        fset=_set_Pj_losses,
        doc=u"""Electrical Joule losses

        :Type: float
        """,
    )

    def _get_Pem_av_ref(self):
        """getter of Pem_av_ref"""
        return self._Pem_av_ref

    def _set_Pem_av_ref(self, value):
        """setter of Pem_av_ref"""
        check_var("Pem_av_ref", value, "float")
        self._Pem_av_ref = value

    Pem_av_ref = property(
        fget=_get_Pem_av_ref,
        fset=_set_Pem_av_ref,
        doc=u"""Average Electromagnetic power

        :Type: float
        """,
    )

    def _get_Us(self):
        """getter of Us"""
        return self._Us

    def _set_Us(self, value):
        """setter of Us"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Us"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataND()
        check_var("Us", value, "DataND")
        self._Us = value

    Us = property(
        fget=_get_Us,
        fset=_set_Us,
        doc=u"""Stator voltage as a function of time (each column correspond to one phase)

        :Type: SciDataTool.Classes.DataND.DataND
        """,
    )

    def _get_internal(self):
        """getter of internal"""
        return self._internal

    def _set_internal(self, value):
        """setter of internal"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "internal"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutInternal()
        check_var("internal", value, "OutInternal")
        self._internal = value

        if self._internal is not None:
            self._internal.parent = self

    internal = property(
        fget=_get_internal,
        fset=_set_internal,
        doc=u"""OutInternal object containg outputs related to a specific model

        :Type: OutInternal
        """,
    )



# -*- coding: utf-8 -*-
from numpy import abs, newaxis, array


def get_cell_area(self, vertices):
    """Return the area of the cells.
    https://en.wikipedia.org/wiki/Shoelace_formula

    Parameters
    ----------
    self : RefTriangle3
        a RefTriangle3 object
    vertices : list or array
        the cells vertices

    Returns
    -------
    area: array
        area of the cells
    """
    if isinstance(vertices, list):
        vertices = array(vertices)

    if vertices.shape[-1] == 2:  # 2D - case
        if len(vertices.shape) == 2:  # only one indice -> adapt array shape
            vertices = vertices[newaxis, :, :]

        a = vertices[:, 0, 0] - vertices[:, 1, 0]  # x1 - x2
        b = vertices[:, 0, 1] - vertices[:, 2, 1]  # y1 - y3
        c = vertices[:, 0, 0] - vertices[:, 2, 0]  # x1 - x3
        d = vertices[:, 0, 1] - vertices[:, 1, 1]  # y1 - y2

    area = 1 / 2 * abs(a * b - c * d)

    return area



# -*- coding: utf-8 -*-

from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve (Line) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotW15
        A SlotW15 object

    Returns
    -------
    curve_list: list
        A list of 6 Segment and 5 Arc

    """

    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]
    Z6 = point_dict["Z6"]
    Z7 = point_dict["Z7"]
    Z8 = point_dict["Z8"]
    Z9 = point_dict["Z9"]
    Z10 = point_dict["Z10"]
    Z11 = point_dict["Z11"]
    Z12 = point_dict["Z12"]
    Z13 = point_dict["Z13"]

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Arc1(Z3, Z4, self.R1))
    curve_list.append(Segment(Z4, Z5))
    curve_list.append(Arc1(Z5, Z6, self.R2))
    curve_list.append(Arc1(Z6, Z8, abs(Z7)))
    curve_list.append(Arc1(Z8, Z9, self.R2))
    curve_list.append(Segment(Z9, Z10))
    curve_list.append(Arc1(Z10, Z11, self.R1))
    curve_list.append(Segment(Z11, Z12))
    curve_list.append(Segment(Z12, Z13))

    return curve_list



# -*- coding: utf-8 -*-


def comp_width_airgap_mag(self):
    """Compute the magnetic airgap (distance beetween the two Lamination)

    Parameters
    ----------
    self: Machine
        A Machine object
    Returns
    -------
    mag_gap: float
        The magnetic airgap [m]

    """

    if self.rotor.is_internal:
        return self.stator.Rint - self.rotor.Rext
    else:
        return self.rotor.Rint - self.stator.Rext



from ....Methods.Geometry.Arc2 import PointTranslateArc2Error


def translate(self, Zt):
    """Translate the Arc2 object with

    Parameters
    ----------
    self : Arc2
        An Arc2 object

    Zt : complex
        Complex coordinates value for translation

    Returns
    -------
    None
    """
    if (
        not isinstance(Zt, complex)
        and not isinstance(Zt, int)
        and not isinstance(Zt, float)
    ):
        raise PointTranslateArc2Error(
            "The point must be a complex number or int or float"
        )

    # check if the Arc2 is correct
    self.check()

    # Modification from the translation of Arc2
    self.begin = self.begin + Zt
    self.center = self.center + Zt



# -*- coding: utf-8 -*-

from numpy import exp


def comp_height_active(self):
    """Compute the height of the winding area

    Parameters
    ----------
    self : SlotCirc
        A SlotCirc object

    Returns
    -------
    Hwind: float
        Height of the winding area [m]
    """

    Rbo = self.get_Rbo()
    alpha = self.comp_angle_opening()

    Z1 = Rbo * exp(-1j * alpha / 2)
    Z2 = Rbo * exp(1j * alpha / 2)
    if self.is_outwards():
        Zbot = (Z1 + Z2) / 2 + self.H0
    else:
        Zbot = (Z1 + Z2) / 2 - self.H0
    return abs(Rbo - abs(Zbot))



# -*- coding: utf-8 -*-

from numpy import pi
from ....Classes.Lamination import Lamination


def comp_volumes(self):
    """Compute the Lamination volumes

    Parameters
    ----------
    self : LamHole
        A LamHole object

    Returns
    -------
    V_dict: dict
        Lamination volume dictionnary (Vlam, Vvent, Vmag, Vhole) [m**3]

    """

    V_dict = Lamination.comp_volumes(self)
    Lt = self.comp_length()

    Vhole = 0
    Vmag = 0
    for hole in self.hole:
        Vhole += hole.Zh * hole.comp_surface() * Lt
        if hole.has_magnet():
            Vmag += hole.Zh * hole.comp_volume_magnets()

    V_dict["Vmag"] = Vmag
    V_dict["Vhole"] = Vhole

    return V_dict



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Geometry/Arc.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Geometry/Arc
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Line import Line

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Geometry.Arc.draw_FEMM import draw_FEMM
except ImportError as error:
    draw_FEMM = error

try:
    from ..Methods.Geometry.Arc.intersect_line import intersect_line
except ImportError as error:
    intersect_line = error

try:
    from ..Methods.Geometry.Arc.is_on_line import is_on_line
except ImportError as error:
    is_on_line = error

try:
    from ..Methods.Geometry.Arc.split_line import split_line
except ImportError as error:
    split_line = error

try:
    from ..Methods.Geometry.Arc.comp_distance import comp_distance
except ImportError as error:
    comp_distance = error

try:
    from ..Methods.Geometry.Arc.plot import plot
except ImportError as error:
    plot = error


from ._check import InitUnKnowClassError


class Arc(Line):
    """Abstract class for arc"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Geometry.Arc.draw_FEMM
    if isinstance(draw_FEMM, ImportError):
        draw_FEMM = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc method draw_FEMM: " + str(draw_FEMM))
            )
        )
    else:
        draw_FEMM = draw_FEMM
    # cf Methods.Geometry.Arc.intersect_line
    if isinstance(intersect_line, ImportError):
        intersect_line = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Arc method intersect_line: " + str(intersect_line)
                )
            )
        )
    else:
        intersect_line = intersect_line
    # cf Methods.Geometry.Arc.is_on_line
    if isinstance(is_on_line, ImportError):
        is_on_line = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc method is_on_line: " + str(is_on_line))
            )
        )
    else:
        is_on_line = is_on_line
    # cf Methods.Geometry.Arc.split_line
    if isinstance(split_line, ImportError):
        split_line = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc method split_line: " + str(split_line))
            )
        )
    else:
        split_line = split_line
    # cf Methods.Geometry.Arc.comp_distance
    if isinstance(comp_distance, ImportError):
        comp_distance = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc method comp_distance: " + str(comp_distance))
            )
        )
    else:
        comp_distance = comp_distance
    # cf Methods.Geometry.Arc.plot
    if isinstance(plot, ImportError):
        plot = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc method plot: " + str(plot))
            )
        )
    else:
        plot = plot
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, label="", init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
        # Set the properties (value check and convertion are done in setter)
        # Call Line init
        super(Arc, self).__init__(label=label)
        # The class is frozen (in Line init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Arc_str = ""
        # Get the properties inherited from Line
        Arc_str += super(Arc, self).__str__()
        return Arc_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Line
        if not super(Arc, self).__eq__(other):
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Line
        diff_list.extend(super(Arc, self).compare(other, name=name))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Line
        S += super(Arc, self).__sizeof__()
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Line
        Arc_dict = super(Arc, self).as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        Arc_dict["__class__"] = "Arc"
        return Arc_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        # Set to None the properties inherited from Line
        super(Arc, self)._set_None()



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Post/PostFunction.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Post/PostFunction
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Post import Post

from ntpath import basename
from os.path import isfile
from ._check import CheckTypeError
import numpy as np
import random
from ._check import InitUnKnowClassError


class PostFunction(Post):
    """Post-processing from a user-defined function"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, run=None, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "run" in list(init_dict.keys()):
                run = init_dict["run"]
        # Set the properties (value check and convertion are done in setter)
        self.run = run
        # Call Post init
        super(PostFunction, self).__init__()
        # The class is frozen (in Post init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        PostFunction_str = ""
        # Get the properties inherited from Post
        PostFunction_str += super(PostFunction, self).__str__()
        if self._run_str is not None:
            PostFunction_str += "run = " + self._run_str + linesep
        elif self._run_func is not None:
            PostFunction_str += "run = " + str(self._run_func) + linesep
        else:
            PostFunction_str += "run = None" + linesep + linesep
        return PostFunction_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Post
        if not super(PostFunction, self).__eq__(other):
            return False
        if other._run_str != self._run_str:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Post
        diff_list.extend(super(PostFunction, self).compare(other, name=name))
        if other._run_str != self._run_str:
            diff_list.append(name + ".run")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Post
        S += super(PostFunction, self).__sizeof__()
        S += getsizeof(self._run_str)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Post
        PostFunction_dict = super(PostFunction, self).as_dict(**kwargs)
        if self._run_str is not None:
            PostFunction_dict["run"] = self._run_str
        elif "keep_function" in kwargs and kwargs["keep_function"]:
            PostFunction_dict["run"] = self.run
        else:
            PostFunction_dict["run"] = None
            if self.run is not None:
                self.get_logger().warning(
                    "PostFunction.as_dict(): "
                    + f"Function {self.run.__name__} is not serializable "
                    + "and will be converted to None."
                )
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        PostFunction_dict["__class__"] = "PostFunction"
        return PostFunction_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.run = None
        # Set to None the properties inherited from Post
        super(PostFunction, self)._set_None()

    def _get_run(self):
        """getter of run"""
        return self._run_func

    def _set_run(self, value):
        """setter of run"""
        if value is None:
            self._run_str = None
            self._run_func = None
        elif isinstance(value, str) and "lambda" in value:
            self._run_str = value
            self._run_func = eval(value)
        elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
            self._run_str = value
            f = open(value, "r")
            exec(f.read(), globals())
            self._run_func = eval(basename(value[:-3]))
        elif callable(value):
            self._run_str = None
            self._run_func = value
        else:
            raise CheckTypeError(
                "For property run Expected function or str (path to python file or lambda), got: "
                + str(type(value))
            )

    run = property(
        fget=_get_run,
        fset=_set_run,
        doc="""Post-processing that takes an output in argument

        :Type: function
        """,
    )



# -*- coding: utf-8 -*-

# File generated according to WHoleMag.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_WHoleMag(object):
    def setupUi(self, WHoleMag):
        if not WHoleMag.objectName():
            WHoleMag.setObjectName(u"WHoleMag")
        WHoleMag.resize(760, 490)
        WHoleMag.setMinimumSize(QSize(760, 490))
        self.main_layout = QVBoxLayout(WHoleMag)
        self.main_layout.setSpacing(4)
        self.main_layout.setObjectName(u"main_layout")
        self.main_layout.setContentsMargins(5, 5, 5, 5)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.c_hole_type = QComboBox(WHoleMag)
        self.c_hole_type.addItem("")
        self.c_hole_type.addItem("")
        self.c_hole_type.addItem("")
        self.c_hole_type.addItem("")
        self.c_hole_type.addItem("")
        self.c_hole_type.addItem("")
        self.c_hole_type.addItem("")
        self.c_hole_type.setObjectName(u"c_hole_type")

        self.horizontalLayout.addWidget(self.c_hole_type)

        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.main_layout.addLayout(self.horizontalLayout)

        self.w_hole = QWidget(WHoleMag)
        self.w_hole.setObjectName(u"w_hole")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.w_hole.sizePolicy().hasHeightForWidth())
        self.w_hole.setSizePolicy(sizePolicy)
        self.w_hole.setMinimumSize(QSize(750, 450))

        self.main_layout.addWidget(self.w_hole)

        self.retranslateUi(WHoleMag)

        QMetaObject.connectSlotsByName(WHoleMag)

    # setupUi

    def retranslateUi(self, WHoleMag):
        WHoleMag.setWindowTitle(QCoreApplication.translate("WHoleMag", u"Form", None))
        self.c_hole_type.setItemText(
            0, QCoreApplication.translate("WHoleMag", u"Slot Type 50", None)
        )
        self.c_hole_type.setItemText(
            1, QCoreApplication.translate("WHoleMag", u"Slot Type 51", None)
        )
        self.c_hole_type.setItemText(
            2, QCoreApplication.translate("WHoleMag", u"Slot Type 52", None)
        )
        self.c_hole_type.setItemText(
            3, QCoreApplication.translate("WHoleMag", u"Slot Type 53", None)
        )
        self.c_hole_type.setItemText(
            4, QCoreApplication.translate("WHoleMag", u"Slot Type 54", None)
        )
        self.c_hole_type.setItemText(
            5, QCoreApplication.translate("WHoleMag", u"Slot Type 55", None)
        )
        self.c_hole_type.setItemText(
            6, QCoreApplication.translate("WHoleMag", u"Slot Type 56", None)
        )

    # retranslateUi



# -*- coding: utf-8 -*-

import numpy as np


def grad_shape_function(self, points):
    """ Return the values of linear shape functions in reference triangle for a given point """

    values = np.zeros([2, 2], dtype=float)
    [x, y] = points
    if (x >= -1) and (x <= 1):
        values[0, 0] = -1 / 2
        values[0, 1] = 1 / 2

    return values



# -*- coding: utf-8 -*-

from numpy import arange

from ....Classes.OutMag import OutMag
from ....Classes.Simulation import Simulation
from ....Classes.ImportMatrixXls import ImportMatrixXls
from ....Methods.Simulation.Input import InputError


def gen_input(self):
    """Generate the input for the structural module (magnetic output)

    Parameters
    ----------
    self : InFlux
        An InFlux object
    """

    output = OutMag()

    # get the simulation
    if isinstance(self.parent, Simulation):
        simu = self.parent
    elif isinstance(self.parent.parent, Simulation):
        simu = self.parent.parent
    else:
        raise InputError(
            "ERROR: InputCurrent object should be inside a Simulation object"
        )

    # Set discretization
    if self.N0 is None:
        if self.OP is None:
            N0 = None  # N0 can be None if time isn't
        else:
            N0 = self.OP.N0
    else:
        N0 = self.N0

    # Import flux components
    per_a = self.per_a
    per_t = self.per_t
    is_antiper_a = self.is_antiper_a
    is_antiper_t = self.is_antiper_t
    out_dict = {}
    for key in self.B_dict:
        comp = self.B_dict[key]
        if isinstance(comp, ImportMatrixXls) and comp.axes_colrows is not None:
            B_comp, axes_values = comp.get_data()
        else:
            B_comp = comp.get_data()
            axes_values = {}
        out_dict[key] = B_comp

    axes_dict = self.comp_axes(
        axes_values,
        N0=N0,
        per_a=per_a,
        is_antiper_a=is_antiper_a,
        per_t=per_t,
        is_antiper_t=is_antiper_t,
    )

    if simu.parent is None:
        raise InputError(
            "ERROR: The Simulation object must be in an Output object to run"
        )
    # Save the Output in the correct place
    if N0 is not None:
        simu.parent.elec.N0 = N0
    output = OutMag()
    output.store(out_dict=out_dict, axes_dict=axes_dict)
    simu.parent.mag = output

    # Define the electrical Output to set the Operating Point
    if self.OP is not None:
        self.OP.gen_input()



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch, Polygon, Wedge
from matplotlib.pyplot import axis, legend
from numpy import array, exp, pi

from ....Functions.init_fig import init_fig
from ....definitions import config_dict

BAR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["BAR_COLOR"]
SCR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["SCR_COLOR"]


def plot(
    self,
    fig=None,
    ax=None,
    is_lam_only=False,
    sym=1,
    alpha=0,
    delta=0,
    is_edge_only=False,
    is_show_fig=True,
):
    """Plot the Lamination in a matplotlib fig

    Parameters
    ----------
    self : LamSquirrelCage
        A LamSquirrelCage object
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        Axis on which to plot the data
    is_lam_only : bool
        True to plot only the lamination (remove the bare)
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_edge_only: bool
        To plot transparent Patches
    is_show_fig : bool
        To call show at the end of the method

    Returns
    -------
    None
    """

    # Lamination and ventilation ducts patches
    (fig, axes, patch_leg, label_leg) = init_fig(fig=fig, ax=ax, shape="rectangle")

    # Plot the lamination
    super(type(self), self).plot(
        fig=fig,
        is_lam_only=is_lam_only,
        sym=sym,
        alpha=alpha,
        delta=delta,
        is_edge_only=is_edge_only,
        is_show_fig=is_show_fig,
    )

    # init figure again to get updated label_leg and patch_leg
    (fig, axes, patch_leg, label_leg) = init_fig(fig)

    # setup the patch of the short circuit ring if needed
    patches = list()
    if not is_lam_only:
        try:
            Rmw = self.slot.comp_radius_mid_active()
            patches.append(
                Wedge(
                    (0, 0),
                    Rmw + self.Hscr / 2.0,
                    0,
                    360,
                    width=self.Hscr,
                    color=SCR_COLOR,
                )
            )  # Full ring
        except:
            pass

    # Display the result
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    axes.set_title("Squirrel Cage Rotor")

    # Axis Setup
    axes.axis("equal")
    Lim = self.Rext * 1.5
    axes.set_xlim(-Lim, Lim)
    axes.set_ylim(-Lim, Lim)

    if not is_lam_only:
        # Add the short ciruit ring to the fig
        for patch in patches:
            axes.add_patch(patch)
        if "Short Circuit Ring" not in label_leg:
            patch_leg.append(Patch(color=SCR_COLOR))
            label_leg.append("Short Circuit Ring")

        legend(patch_leg, label_leg)
    if is_show_fig:
        fig.show()



import numpy as np
import matplotlib.pyplot as plt
from ....Classes.OptiObjective import OptiObjective
from ....Methods.Output.XOutput import _get_symbol_data_


def plot_pareto(
    self,
    x_symbol,
    y_symbol,
    c_symbol=None,
    cmap=None,
    ax=None,
    title=None,
    grid=False,
    is_show_fig=True,
    save_path=None,
):
    """Plot the pareto front for 2 objective functions

    Parameters
    ----------
    self : XOutput
    x_symbol : str
        symbol of the first objective function
    y_symbol: str
        symbol of the second objective function
    c_symbol: str
        optional symbol to set the plot colors
    cmap: colormap
        optional colormap
    is_show_fig : bool
        True to show figure after plot
    save_path : str
        full path of the png file where the figure is saved if save_path is not None
    """

    # Pyleecan colors
    pyleecan_color = (230 / 255, 175 / 255, 0)

    # Gather fitness results
    data = [
        val.result
        for _, val in self.xoutput_dict.items()
        if isinstance(val, OptiObjective)
    ]
    fitness = np.array(data).T

    # Get fitness values and ngen
    is_valid = np.array(self["is_valid"].result)
    # ngen = np.array(self["ngen"].result)  # unused

    design_var_list = [pe.value for pe in self.paramexplorer_list]
    design_var = np.array(design_var_list).T

    # Keep only valid values
    indx = np.where(is_valid)
    fitness = fitness[indx]
    # ngen = ngen[indx]   # unused
    design_var = design_var[indx]

    # get data and labels
    x_values, x_label = _get_symbol_data_(self, x_symbol, indx)
    y_values, y_label = _get_symbol_data_(self, y_symbol, indx)

    # Get pareto front
    pareto = fitness

    # Get dominated values
    idx_non_dom = list(range(len(pareto)))
    N = len(pareto)
    for i in range(N):
        for j in idx_non_dom:
            if all(pareto[j] <= pareto[i]) and any(pareto[j] < pareto[i]):
                idx_non_dom.remove(i)
                break

    pareto = pareto[idx_non_dom]
    design_var_values = design_var[idx_non_dom]

    # Write annotations
    legend_annot = []

    design_var_symbols = [pe.symbol for pe in self.paramexplorer_list]

    for idx, sim in enumerate(design_var_values.tolist()):
        legend = f"Individual Nr. {indx[0][idx_non_dom[idx]]}\n"
        for ii, symbol in enumerate(design_var_symbols):
            legend += "{:11.10}=".format(symbol)  # sim[d_var])
            if isinstance(sim[ii], float):
                legend += " {:3.3e}\n".format(sim[ii])
            else:
                legend += "{:>11.10}\n".format(str(sim[ii]))
        legend_annot.append(legend[:-1])
    if ax is None:
        fig, ax = plt.subplots()
        return_ax = False
    else:
        return_ax = True
        fig = ax.get_figure()

    if c_symbol is None:
        colors = pyleecan_color
    else:
        # get the color data
        c_values, _ = _get_symbol_data_(self, c_symbol, indx)
        colors = c_values[idx_non_dom][:, np.newaxis]

    if cmap is None:
        cmap = plt.cm.jet

    # Plot Pareto front
    sc = ax.scatter(
        x_values[idx_non_dom][:, np.newaxis],
        y_values[idx_non_dom][:, np.newaxis],
        # facecolors=colors,
        c=colors,
        edgecolors=(0.35, 0.35, 0.35),
        label="Pareto Front",
        cmap=cmap,
    )
    # Add legend
    if c_symbol is not None:
        legend1 = ax.legend(*sc.legend_elements(), loc="upper right", title=c_symbol)
        ax.add_artist(legend1)

    ax.autoscale(1, 1)

    ax.set_title("Pareto Front")
    ax.set_xlabel(x_label)
    ax.set_ylabel(y_label)

    if grid:
        ax.set_axisbelow(True)
        ax.grid()

    # Add anotations in the plot see https://stackoverflow.com/a/47166787
    annot = ax.annotate(
        "",
        xy=(0, 0),
        xytext=(20, 20),
        textcoords="offset points",
        bbox=dict(boxstyle="round", fc="w"),
        arrowprops=dict(arrowstyle="->"),
    )
    annot.set_visible(False)

    def update_annot(ind):
        """ Update annotation """
        # Get ind position
        pos = sc.get_offsets()[ind["ind"][0]]
        annot.xy = pos

        # Set the annotation
        annot.set_text(legend_annot[ind["ind"][0]])
        annot.get_bbox_patch().set_facecolor(pyleecan_color)
        annot.get_bbox_patch().set_alpha(0.4)

    def hover(event):
        # Check if annotation is visible
        vis = annot.get_visible()

        if event.inaxes == ax:
            cont, ind = sc.contains(event)
            if cont:
                update_annot(ind)
                annot.set_visible(True)
                fig.canvas.draw_idle()
            else:
                if vis:
                    annot.set_visible(False)
                    fig.canvas.draw_idle()

    fig.canvas.mpl_connect("motion_notify_event", hover)

    if save_path is not None:
        fig.savefig(save_path)
        plt.close()

    if is_show_fig:
        fig.show()

    if return_ax:
        return ax
    else:
        fig.show()



# -*- coding: utf-8 -*-

import PySide2.QtCore
from numpy import pi
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QWidget

from ......Classes.SlotM15 import SlotM15
from ......GUI import gui_option
from ......GUI.Dialog.DMachineSetup.SMSlot.PMSlot15.Gen_PMSlot15 import Gen_PMSlot15
from ......Methods.Slot.Slot import SlotCheckError

translate = PySide2.QtCore.QCoreApplication.translate


class PMSlot15(Gen_PMSlot15, QWidget):
    """Page to set the Slot Magnet Type 14"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for Slot combobox
    slot_name = "Polar Magnet with curved top and parallel sides"
    slot_type = SlotM15

    def __init__(self, lamination=None):
        """Initialize the widget according to lamination

        Parameters
        ----------
        self : PMSlot15
            A PMSlot15 widget
        lamination : Lamination
            current lamination to edit
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)
        self.lamination = lamination
        self.slot = lamination.slot

        # Set FloatEdit unit
        self.lf_W0.unit = "rad"
        self.lf_Wmag.unit = "m"
        self.lf_H0.unit = "m"
        self.lf_Hmag.unit = "m"
        self.lf_Rtopm.unit = "m"
        # Set unit name (m ou mm)
        wid_list = [
            self.unit_H0,
            self.unit_Hmag,
            self.unit_Rtopm,
        ]
        for wid in wid_list:
            wid.setText("[" + gui_option.unit.get_m_name() + "]")

        # Fill the fields with the machine values (if they're filled)
        self.lf_W0.setValue(self.slot.W0)
        self.lf_Wmag.setValue(self.slot.Wmag)
        self.lf_H0.setValue(self.slot.H0)
        self.lf_Hmag.setValue(self.slot.Hmag)
        self.lf_Rtopm.setValue(self.slot.Rtopm)

        # Display the main output of the slot (surface, height...)
        self.w_out.comp_output()

        # Connect the signal
        self.lf_W0.editingFinished.connect(self.set_W0)
        self.lf_Wmag.editingFinished.connect(self.set_Wmag)
        self.lf_H0.editingFinished.connect(self.set_H0)
        self.lf_Hmag.editingFinished.connect(self.set_Hmag)
        self.lf_Rtopm.editingFinished.connect(self.set_Rtopm)

    def set_W0(self):
        """Signal to update the value of W0 according to the line edit

        Parameters
        ----------
        self : PMSlot15
            A PMSlot15 object
        """
        self.slot.W0 = self.lf_W0.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Wmag(self):
        """Signal to update the value of Wmag according to the line edit

        Parameters
        ----------
        self : PMSlot15
            A PMSlot15 object
        """
        self.slot.Wmag = self.lf_Wmag.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_H0(self):
        """Signal to update the value of H0 according to the line edit

        Parameters
        ----------
        self : PMSlot15
            A PMSlot15 object
        """
        self.slot.H0 = self.lf_H0.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Hmag(self):
        """Signal to update the value of Hmag according to the line edit

        Parameters
        ----------
        self : PMSlot15
            A PMSlot15 object
        """
        self.slot.Hmag = self.lf_Hmag.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Rtopm(self):
        """Signal to update the value of Rtopm according to the line edit

        Parameters
        ----------
        self : PMSlot15
            A PMSlot15 object
        """
        self.slot.Rtopm = self.lf_Rtopm.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    @staticmethod
    def check(lam):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        lam: LamSlotMag
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """

        # Check that everything is set
        if lam.slot.W0 is None:
            return "You must set W0 !"
        elif lam.slot.Wmag is None:
            return "You must set Wmag !"
        elif lam.slot.H0 is None:
            return "You must set H0 !"
        elif lam.slot.Hmag is None:
            return "You must set Hmag !"
        elif lam.slot.Rtopm is None:
            return "You must set Rtopm !"

        # Constraints
        try:
            lam.slot.check()
        except SlotCheckError as error:
            return str(error)

        # Output
        try:
            yoke_height = lam.comp_height_yoke()
        except Exception as error:
            return translate("Unable to compute yoke height:", "PMSlot15") + str(error)

        if yoke_height <= 0:
            return translate(
                "The slot height is greater than the lamination !", "PMSlot15"
            )



# -*- coding: utf-8 -*-

from numpy import cos, exp
from numpy import abs as np_abs


def comp_radius(self):
    """Compute the radius of the min and max circle that contains the hole

    Parameters
    ----------
    self : Hole
        A Hole object

    Returns
    -------
    (Rmin,Rmax): tuple
        Radius of the circle that contains the hole

    """

    surf_list = self.build_geometry()
    point_list = list()
    for surf in surf_list:
        for curve in surf.get_lines():
            point_list.extend(curve.discretize())

    abs_list = [np_abs(point) for point in point_list]
    Rmax = max(abs_list)
    Rmin = min(abs_list)
    return (Rmin, Rmax)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Elmer/SolverInputFile.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Elmer/SolverInputFile
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Elmer import Elmer

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Elmer.SolverInputFile.write import write
except ImportError as error:
    write = error


from ._check import InitUnKnowClassError


class SolverInputFile(Elmer):
    """Class to setup the Elmer Solver Input File"""

    VERSION = 1

    # cf Methods.Elmer.SolverInputFile.write
    if isinstance(write, ImportError):
        write = property(
            fget=lambda x: raise_(
                ImportError("Can't use SolverInputFile method write: " + str(write))
            )
        )
    else:
        write = write
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, sections=-1, logger_name="Pyleecan.Elmer", init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "sections" in list(init_dict.keys()):
                sections = init_dict["sections"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.sections = sections
        # Call Elmer init
        super(SolverInputFile, self).__init__(logger_name=logger_name)
        # The class is frozen (in Elmer init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        SolverInputFile_str = ""
        # Get the properties inherited from Elmer
        SolverInputFile_str += super(SolverInputFile, self).__str__()
        SolverInputFile_str += (
            "sections = "
            + linesep
            + str(self.sections).replace(linesep, linesep + "\t")
            + linesep
        )
        return SolverInputFile_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Elmer
        if not super(SolverInputFile, self).__eq__(other):
            return False
        if other.sections != self.sections:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Elmer
        diff_list.extend(super(SolverInputFile, self).compare(other, name=name))
        if other._sections != self._sections:
            diff_list.append(name + ".sections")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Elmer
        S += super(SolverInputFile, self).__sizeof__()
        if self.sections is not None:
            for value in self.sections:
                S += getsizeof(value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Elmer
        SolverInputFile_dict = super(SolverInputFile, self).as_dict(**kwargs)
        SolverInputFile_dict["sections"] = (
            self.sections.copy() if self.sections is not None else None
        )
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        SolverInputFile_dict["__class__"] = "SolverInputFile"
        return SolverInputFile_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.sections = None
        # Set to None the properties inherited from Elmer
        super(SolverInputFile, self)._set_None()

    def _get_sections(self):
        """getter of sections"""
        return self._sections

    def _set_sections(self, value):
        """setter of sections"""
        if type(value) is int and value == -1:
            value = list()
        check_var("sections", value, "list")
        self._sections = value

    sections = property(
        fget=_get_sections,
        fset=_set_sections,
        doc=u"""List of SIF sections

        :Type: list
        """,
    )



# -*- coding: utf-8 -*-

from os import getcwd, rename
from os.path import basename, join, isfile, dirname

from PySide2.QtCore import Qt, Signal
from PySide2.QtWidgets import QFileDialog, QMessageBox, QWidget

from ....Functions.Material.compare_material import compare_material
from ....Functions.load import load, load_matlib
from ....GUI.Dialog.DMachineSetup import mach_index, mach_list
from ....GUI.Dialog.DMachineSetup.Ui_DMachineSetup import Ui_DMachineSetup
from ....GUI.Dialog.DMachineSetup.SPreview.SPreview import SPreview
from ....definitions import config_dict
from ....Classes.Machine import Machine
from ....Classes.Material import Material
from logging import getLogger

# Flag for set the enable property of w_nav (List_Widget)
DISABLE_ITEM = Qt.NoItemFlags
ENABLE_ITEM = Qt.ItemIsSelectable | Qt.ItemIsEnabled


class DMachineSetup(Ui_DMachineSetup, QWidget):
    """Main windows of the Machine Setup Tools"""

    # Signal to update the simulation
    machineChanged = Signal()
    rejected = Signal()

    def __init__(self, machine=None, dmatlib=None, machine_path=""):
        """Initialize the GUI according to machine type

        Parameters
        ----------
        self : DMachineSetup
            a DMachineSetup object
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        self.is_save_needed = False
        self.dmatlib = dmatlib
        self.matlib = dmatlib.matlib
        self.last_index = 0  # Index of the last step available

        # Saving arguments
        self.machine = machine
        if machine_path == "":
            self.machine_path = config_dict["MAIN"]["MACHINE_DIR"]
        else:
            self.machine_path = machine_path

        # Initialize the machine if needed
        if machine is None:
            self.machine = type(mach_list[0]["init_machine"])(
                init_dict=mach_list[0]["init_machine"].as_dict()
            )

        self.update_nav()
        self.set_nav(self.last_index)

        # Connect save/load button
        self.nav_step.currentRowChanged.connect(self.set_nav)
        self.b_save.clicked.connect(self.s_save)
        self.b_load.clicked.connect(self.s_load)

        self.dmatlib.saveNeeded.connect(self.save_needed)

        self.qmessagebox_question = None

    def save_needed(self):
        """Set is_save_needed to True"""
        self.is_save_needed = True

    def closeEvent(self, event):
        """Display a message before leaving

        Parameters
        ----------
        self : DMachineSetup
            A DMachineSetup object
        event :
            The closing event
        """

        if self.is_save_needed:
            quit_msg = self.tr(
                "Unsaved changes will be lost.\nDo you want to save the machine?"
            )
            reply = QMessageBox.question(
                self,
                self.tr("Please save before closing"),
                quit_msg,
                QMessageBox.Yes,
                QMessageBox.No,
            )
            self.qmessagebox_question = reply
            if reply == QMessageBox.Yes:
                self.s_save()

    def reject(self):
        """ """
        self.rejected.emit()
        self.close()

    def s_save(self):
        """Slot for saving the current machine to a json file

        Parameters
        ----------
        self : DMachineSetup
            A DMachineSetup object
        """
        # Ask the user to select a .m file to save
        if self.machine.name in ["", None]:
            save_file_path = QFileDialog.getSaveFileName(
                self, self.tr("Save file"), self.machine_path, "Json (*.json)"
            )[0]
        else:
            def_path = join(self.machine_path, self.machine.name + ".json")
            save_file_path = QFileDialog.getSaveFileName(
                self, self.tr("Save file"), def_path, "Json (*.json)"
            )[0]

        # Avoid bug due to user closing the popup witout selecting a file
        if save_file_path != "":
            # Set the machine name to match the file name
            self.machine.name = str(basename(str(save_file_path)))[:-5]
            # Save the machine file
            self.machine.save(save_file_path)
            # To update the machine name field (if first page)
            self.set_nav(self.nav_step.currentRow())
            # Update the machine path to remember the last used folder
            self.machine_path = dirname(save_file_path)
            # Notify the project GUI that the machine has changed
            self.machineChanged.emit()
            self.is_save_needed = False
            return True
        return False

    def s_save_close(self):
        """Signal to save and close

        Parameters
        ----------
        self : DMachineSetup
            A DMachineSetup object
        """
        to_close = self.s_save()
        if to_close:
            self.close()

    def s_load(self):
        """Slot to load a machine from a .json file (triggered by b_load)

        Parameters
        ----------
        self : DMachineSetup
            A DMachineSetup object
        """
        ### TODO: handle material data, i.e. "connect", set new material, etc.

        # Ask the user to select a .json file to load
        load_path = str(
            QFileDialog.getOpenFileName(
                self, self.tr("Load file"), self.machine_path, "Json (*.json)"
            )[0]
        )
        if load_path != "":
            try:
                # Update the machine path to remember the last used folder
                self.machine_path = dirname(load_path)
                # Load and check type of instance
                machine = load(load_path)
                if isinstance(machine, Machine):
                    self.machine = machine
                    is_machine_change = self.matlib.add_machine_mat(machine)
                else:
                    QMessageBox().critical(
                        self,
                        self.tr("Error"),
                        self.tr("The choosen file is not a machine file."),
                    )
                    return
                self.machineChanged.emit()

                # Save needed if machine materials have changed
                if is_machine_change:
                    self.save_needed()

            except Exception as e:
                QMessageBox().critical(
                    self,
                    self.tr("Error"),
                    self.tr(
                        "The machine file is incorrect:\n",
                        "Please keep the \n, another " "message is following this one",
                    )
                    + type(e).__name__
                    + ": "
                    + str(e),
                )
                return
            self.update_nav()

    def update_nav(self, next_step=None):
        """Update the nav list to match the step of the current machine"""
        mach_dict = mach_list[self.get_machine_index()]
        self.nav_step.blockSignals(True)
        self.nav_step.clear()
        index = 1
        for step in mach_dict["start_step"]:
            self.nav_step.addItem(" " + str(index) + ": " + step.step_name)
            index += 1
        for step in mach_dict["stator_step"]:
            self.nav_step.addItem(" " + str(index) + ": Stator " + step.step_name)
            index += 1
        for step in mach_dict["rotor_step"]:
            if index < 10:
                self.nav_step.addItem(" " + str(index) + ": Rotor " + step.step_name)
            else:
                self.nav_step.addItem(str(index) + ": Rotor " + step.step_name)
            index += 1
        # Adding last step Machine Summary
        if index < 10:
            self.nav_step.addItem(" " + str(index) + ": " + SPreview.step_name)
        else:
            self.nav_step.addItem(str(index) + ": " + SPreview.step_name)
        self.update_enable_nav()
        self.nav_step.blockSignals(False)
        if next_step is None:
            self.nav_step.setCurrentRow(self.last_index)
        else:
            self.nav_step.setCurrentRow(next_step)

    def update_enable_nav(self):
        # Load for readibility
        nav = self.nav_step
        machine = self.machine
        mach_dict = mach_list[self.get_machine_index()]
        # First we disable all the item in the navigation widget
        for ii in range(0, nav.count()):
            nav.item(ii).setFlags(DISABLE_ITEM)
        # First step is always available
        nav.item(0).setFlags(ENABLE_ITEM)
        index = 1
        # Check the start steps
        for step in mach_dict["start_step"]:
            if step.check(machine) is not None:
                self.last_index = index - 1
                return None  # Exit at the first fail
            nav.item(index).setFlags(ENABLE_ITEM)
            index += 1
        # Check the stator steps
        for step in mach_dict["stator_step"]:
            if step.check(machine.stator) is not None:
                self.last_index = index - 1
                return None  # Exit at the first fail
            nav.item(index).setFlags(ENABLE_ITEM)
            index += 1
        # Check the rotor steps
        for step in mach_dict["rotor_step"]:
            if step.check(machine.rotor) is not None:
                self.last_index = index - 1
                return None  # Exit at the first fail
            nav.item(index).setFlags(ENABLE_ITEM)
            index += 1
        self.last_index = index - 1

    def get_machine_index(self):
        """Get the index corresponding to the current machine in the mach_list"""
        # Get the correct machine dictionnary
        index = mach_index.index(type(self.machine))
        if index == -1:
            QMessageBox().critical(
                self, self.tr("Error"), self.tr("Unknown machine type")
            )
            self.machine = type(mach_list[0]["init_machine"])(
                init_dict=mach_list[0]["init_machine"].as_dict()
            )
            return self.get_machine_index()
        return index

    def set_nav(self, index):
        """Select the current widget according to the current machine type
        and the current nav index

        Parameters
        ----------
        self : DMachineSetup
            A DMachineSetup object
        index : int
            Current index of nav_step
        """
        # Get the step list of the current machine
        mach_dict = mach_list[self.get_machine_index()]
        step_list = list()
        step_list.extend(mach_dict["start_step"])
        step_list.extend(mach_dict["stator_step"])
        step_list.extend(mach_dict["rotor_step"])
        step_list.append(SPreview)
        is_stator = "Stator" in self.nav_step.currentItem().text()

        # Regenerate the step with the current values
        self.w_step.setParent(None)
        self.w_step = step_list[index](
            machine=self.machine, matlib=self.matlib, is_stator=is_stator
        )
        self.w_step.b_previous.clicked.connect(self.s_previous)
        if index != len(step_list) - 1:
            self.w_step.b_next.setText(self.tr(u"Next"))
            self.w_step.b_next.clicked.connect(self.s_next)
        else:
            self.w_step.b_next.setText(self.tr(u"Save and Close"))
            self.w_step.b_next.clicked.connect(self.s_save_close)

        self.w_step.saveNeeded.connect(self.save_needed)
        # Refresh the GUI
        self.main_layout.insertWidget(1, self.w_step)

    def s_next(self):
        """Signal to set the next step as accessible (and selected)

        Parameters
        ----------
        self : DMachineSetup
            A DMachineSetup object
        """
        next_index = self.nav_step.currentRow() + 1
        mach_dict = mach_list[self.get_machine_index()]

        if next_index - 1 < len(mach_dict["start_step"]):
            error = self.w_step.check(self.machine)
        elif next_index - 1 < len(mach_dict["start_step"]) + len(
            mach_dict["stator_step"]
        ):
            error = self.w_step.check(self.machine.stator)
        else:
            error = self.w_step.check(self.machine.rotor)

        if error is not None:  # An error: display it in a popup
            QMessageBox().critical(self, self.tr("Error"), error)
        else:  # No error => Go to the next page
            self.nav_step.item(next_index).setFlags(ENABLE_ITEM)
            self.last_index = next_index
            self.nav_step.setCurrentRow(next_index)
            # As the current row have changed, set_nav is called

    def s_previous(self):
        """Signal to set the previous page of w_page_stack as accessible (and selected)

        Parameters
        ----------
        self : DMachineSetup
            A DMachineSetup object
        """
        next_index = self.nav_step.currentRow() - 1
        self.nav_step.setCurrentRow(next_index)
        # As the current row have change, set_nav is called



# -*- coding: utf-8 -*-

from numpy import pi
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QMessageBox, QWidget

from .....Classes.LamSlotWind import LamSlotWind
from .....Classes.Slot import Slot
from .....Classes.SlotW60 import SlotW60
from .....Classes.SlotW61 import SlotW61
from .....Classes.Slot import Slot
from .....GUI.Dialog.DMachineSetup.SWPole.PWSlot60.PWSlot60 import PWSlot60
from .....GUI.Dialog.DMachineSetup.SWPole.PWSlot61.PWSlot61 import PWSlot61
from .....GUI.Dialog.DMachineSetup.SWPole.Ui_SWPole import Ui_SWPole

# List to convert index of combobox to slot type
INIT_INDEX = [SlotW60, SlotW61]
NAME_INDEX = [slot.__name__ for slot in INIT_INDEX]
WIDGET_LIST = [PWSlot60, PWSlot61]


class SWPole(Ui_SWPole, QWidget):
    """Step to set the lamination pole (for WRSM)"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for DMachineSetup
    step_name = "Pole"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the GUI according to machine

        Parameters
        ----------
        self : SWPole
            A SWPole widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        self.b_help.hide()

        # Avoid erase all the parameters when navigating though the slots
        self.previous_slot = {SlotW60: None, SlotW61: None}

        if self.is_stator:
            self.obj = machine.stator
        else:
            self.obj = machine.rotor

        # If the Slot is not set, initialize it with a SlotW60
        if self.obj.slot is None or type(self.obj.slot) not in INIT_INDEX:
            self.obj.slot = SlotW60()
            self.obj.slot._set_None()

        # Avoid error when loading WRSM with wrong rotor slot
        if type(self.obj.slot) not in INIT_INDEX:
            self.obj.slot = SlotW60()
            self.obj.slot._set_None()

        self.obj.slot.Zs = self.obj.winding.p * 2
        self.update_slot_text(self.obj.slot.Zs)

        # Set the correct index for the type checkbox
        index = NAME_INDEX.index(type(self.obj.slot).__name__)
        self.c_slot_type.setCurrentIndex(index)

        # Update the slot widget
        self.s_update_slot()

        # Connect the slot/signal
        self.c_slot_type.currentIndexChanged.connect(self.s_change_slot)
        self.b_plot.clicked.connect(self.s_plot)

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()

    def set_slot_type(self, index):
        """Initialize self.obj with the slot corresponding to index

        Parameters
        ----------
        self : SWPole
            A SWPole object
        index : int
            Index of the selected slot type in the list
        """

        # Save the slot
        self.previous_slot[type(self.obj.slot)] = self.obj.slot

        # Call the corresponding constructor
        Zs = self.obj.slot.Zs
        if self.previous_slot[INIT_INDEX[index]] is None:
            # No previous slot of this type
            self.obj.slot = INIT_INDEX[index]()
            self.obj.slot._set_None()  # No default value
            self.obj.slot.Zs = Zs
        else:  # Load the previous slot of this type
            self.obj.slot = self.previous_slot[INIT_INDEX[index]]

        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def update_slot_text(self, Zs):
        """Update in_Zs and out_slot_pitch with the correct value

        Parameters
        ----------
        self : SWPole
            A SWPole object
        Zs : int
            The current value of Zs
        """
        sp_txt = self.tr("Slot pitch = 360 / Zs = ")
        self.in_Zs.setText("Zs = 2*p = " + str(Zs))

        if Zs in [None, 0]:
            self.out_Slot_pitch.setText(sp_txt + "?")
        else:
            Slot_pitch = 360.0 / Zs
            Slot_pitch_rad = Slot_pitch * pi / 180

            self.out_Slot_pitch.setText(
                sp_txt
                + "%.4g" % (Slot_pitch)
                + u" ° ("
                + "%.4g" % (Slot_pitch_rad)
                + " rad)"
            )

    def s_update_slot(self):
        """Update the slot widget

        Parameters
        ----------
        self : SWPole
            a SWPole object
        """

        # Regenerate the pages with the new values
        self.w_slot.setParent(None)
        self.w_slot = WIDGET_LIST[self.c_slot_type.currentIndex()](self.obj)
        self.w_slot.saveNeeded.connect(self.emit_save)
        # Refresh the GUI
        self.main_layout.removeWidget(self.w_slot)
        self.main_layout.insertWidget(1, self.w_slot)

    def s_change_slot(self, index):
        """Signal to update the interface and show a specific page

        Parameters
        ----------
        self : SWPole
            a SWPole object
        index : int
            Index of w_page_stack to show
        """
        # Current slot is removed and replaced by the new one
        self.set_slot_type(index)
        self.s_update_slot()

    def s_plot(self):
        """Try to plot the lamination

        Parameters
        ----------
        self : SWPole
            A SWPole object
        """
        # We have to make sure the slot is right before truing to plot it
        error = self.check(self.obj)

        if error:  # Error => Display it
            QMessageBox().critical(self, self.tr("Error"), error)
        else:  # No error => Plot the slot (No winding for LamSquirrelCage)
            self.obj.plot(plot_activeing=(type(self.obj) is LamSlotWind))

    @staticmethod
    def check(lam):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        lam : Lamination
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)

        """

        # Call the check method of the slot (every slot type have a
        # different check method)
        try:
            index = INIT_INDEX.index(type(lam.slot))
            return WIDGET_LIST[index].check(lam)
        except Exception as e:
            return str(e)



# -*- coding: utf-8 -*-

from numpy import exp, pi


def get_center(self):
    """Return a list of center of the ventilations
    Parameters
    ----------
    self : VentilationTrap
        A VentilationTrap object

    Returns
    -------
    Zc_list: list
        List of list of center complex coordinates

    """

    Zc_list = list()

    for ii in range(self.Zh):
        Zc = (
            (self.H0 + self.D0 / 2.0)
            * exp(1j * self.Alpha0)
            * exp(ii * 1j * 2 * pi / self.Zh)
        )
        Zc_list.append(Zc)

    return Zc_list



# -*- coding: utf-8 -*-


def get_polar_eq(self):
    """Returns a polar equivalent of the machine

    Parameters
    ----------
    self : Machine
        Machine object

    Returns
    -------
    polar_eq: Machine
        The polar equivalent of the machine
    """

    # Copy the machine
    polar_eq = type(self)(init_dict=self.as_dict())

    polar_eq.rotor = polar_eq.rotor.get_polar_eq()
    polar_eq.stator = polar_eq.stator.get_polar_eq()
    # TODO: polar frame
    return polar_eq



from numpy import exp, angle, abs as np_abs
from ....Classes.Segment import Segment
from ....definitions import PACKAGE_NAME

DELTA = 1e-9  # To remove computing noise


def split_line(self, Z1, Z2, is_top=True, is_join=False, label_join=""):
    """Cut the Arc according to a line defined by two complex

    Parameters
    ----------
    self : Segment
        An Segment object
    Z1 : complex
        First point of the cutting Line
    Z2 : complex
        Second point of the cutting Line
    is_top : bool
        True to keep the part above the cutting line.
        "Above" is in the coordinate system with Z1 in 0 and Z2 on the X>0 axis
    is_join : bool
        True to join the split_list with Segment if there is more that one remaining parts
    label_join : str
        Label of the join line

    Returns
    -------
    split_list : list(Arc)
        The selected part of the arc (0, 1 or 2 arc depending on cutting point)
    """

    # Dynamic import of Arc1 to prevent import loop from code generator
    module = __import__(PACKAGE_NAME + ".Classes.Arc1", fromlist=["Arc1"])
    Arc1 = getattr(module, "Arc1")

    # Get the intersection point
    Z_int = self.intersect_line(Z1, Z2)
    # Begin and end point in the line coordonate system
    Zb = (self.get_begin() - Z1) * exp(-1j * angle(Z2 - Z1))
    Ze = (self.get_end() - Z1) * exp(-1j * angle(Z2 - Z1))
    # Center of the arc (for later check)
    Zc = self.get_center()

    if len(Z_int) == 0:
        # No intersection copy the line
        line = type(self)(init_dict=self.as_dict())
        if Zb.imag >= 0 and Ze.imag >= 0 and is_top:
            return [line]
        if Zb.imag >= 0 and Ze.imag >= 0 and not is_top:
            return list()
        if Zb.imag <= 0 and Ze.imag <= 0 and is_top:
            return list()
        return [line]
    if len(Z_int) == 1:
        # One intersection => Three possible lines
        # Begin => Intersection
        if np_abs(Z_int[0] - self.get_begin()) > DELTA:
            line1 = Arc1(
                begin=self.get_begin(),
                end=Z_int[0],
                radius=self.comp_radius(),
                is_trigo_direction=bool(self.get_angle() > 0),
            )
            # Fix radius if needed
            if np_abs(line1.get_center() - Zc) > 1e-6:
                line1.radius = -1 * line1.radius
        else:  # Begin == Intersection
            line1 = None
        # Intersection => End
        if np_abs(Z_int[0] - self.get_end()) > DELTA:
            line2 = Arc1(
                begin=Z_int[0],
                end=self.get_end(),
                radius=self.comp_radius(),
                is_trigo_direction=bool(self.get_angle() > 0),
            )
            # Fix radius if needed
            if np_abs(line2.get_center() - Zc) > 1e-6:
                line2.radius = -1 * line2.radius
        else:  # Intersection == End
            line2 = None
        # Copy of the complete line (begin or end on cutting line, or cutting line is tangent)
        line3 = type(self)(init_dict=self.as_dict())

        # If the line is tangent, begin and end are on the same side of the line
        if (Zb.imag > DELTA and Ze.imag > DELTA) or (
            Zb.imag < -DELTA and Ze.imag < -DELTA
        ):
            if Zb.imag > DELTA and is_top:
                return [line3]
            if Zb.imag > DELTA and not is_top:
                return []
            if Zb.imag < -DELTA and is_top:
                return []
            if Zb.imag < -DELTA and not is_top:
                return [line3]
        # Return the correct line according to the points position
        if Zb.imag > DELTA and is_top:
            if line1:
                return [line1]
            return []
        if Zb.imag > DELTA and not is_top:
            if line2:
                return [line2]
            return []
        if Zb.imag < -DELTA and is_top:
            if line2:
                return [line2]
            return []
        if Zb.imag < -DELTA and not is_top:
            if line1:
                return [line1]
            return []
        # Zb.imag == 0 => begin on cutting line
        if Ze.imag > DELTA and is_top:
            return [line3]
        if Ze.imag > DELTA and not is_top:
            return []
        if Ze.imag < -DELTA and is_top:
            return []
        if Ze.imag < -DELTA and not is_top:
            return [line3]
    if len(Z_int) == 2:
        # Two intersection => Three possible lines
        # Begin => Intersection1 (intersection are ordered along the arc)
        if np_abs(Z_int[0] - self.get_begin()) > DELTA:
            line1 = Arc1(
                begin=self.get_begin(),
                end=Z_int[0],
                radius=self.comp_radius(),
                is_trigo_direction=bool(self.get_angle() > 0),
            )
            # Fix radius if needed
            if np_abs(line1.get_center() - Zc) > 1e-6:
                line1.radius = -1 * line1.radius
        else:  # Begin == Intersection 1
            line1 = None
        # Intersection 1 => Intersection 2
        line2 = Arc1(
            begin=Z_int[0],
            end=Z_int[1],
            radius=self.comp_radius(),
            is_trigo_direction=bool(self.get_angle() > 0),
        )
        # Fix radius if needed
        if np_abs(line2.get_center() - Zc) > 1e-6:
            line2.radius = -1 * line2.radius
        # Intersection 2 => End
        if np_abs(Z_int[1] - self.get_end()) > DELTA:
            line3 = Arc1(
                begin=Z_int[1],
                end=self.get_end(),
                radius=self.comp_radius(),
                is_trigo_direction=bool(self.get_angle() > 0),
            )
            # Fix radius if needed
            if np_abs(line3.get_center() - Zc) > 1e-6:
                line3.radius = -1 * line3.radius
        else:  # Intersection == End
            line3 = None
        # Line Intersection 1 => Intersection 2 for join
        seg_join = Segment(begin=Z_int[0], end=Z_int[1], label=label_join)

        # If the intersetion points are begin and end
        if (
            np_abs(Z_int[0] - self.get_begin()) < DELTA
            and np_abs(Z_int[1] - self.get_end()) < DELTA
        ):
            Zm = (self.get_middle() - Z1) * exp(-1j * angle(Z2 - Z1))
            # begin and end are on the line => Zm.imag != 0
            if Zm.imag > 0 and is_top:
                return [type(self)(init_dict=self.as_dict())]
            elif Zm.imag > 0 and not is_top:
                return []
            elif Zm.imag < 0 and is_top:
                return []
            elif Zm.imag < 0 and not is_top:
                return [type(self)(init_dict=self.as_dict())]

        # Return the correct line(s) according to the points position
        line_list = list()
        if Zb.imag > DELTA and is_top:
            if line1:
                line_list.append(line1)
            if line3:
                line_list.append(line3)
            if len(line_list) == 2 and is_join:
                return [line1, seg_join, line3]
            else:
                return line_list
        if Zb.imag > DELTA and not is_top:
            return [line2]
        if Zb.imag < -DELTA and is_top:
            return [line2]
        if Zb.imag < -DELTA and not is_top:
            if line1:
                line_list.append(line1)
            if line3:
                line_list.append(line3)
            if len(line_list) == 2 and is_join:
                return [line1, seg_join, line3]
            else:
                return line_list
        # Begin on cutting line
        if Ze.imag > DELTA and is_top:
            return [line3]
        if Ze.imag > DELTA and not is_top:
            return [line2]
        if Ze.imag < -DELTA and is_top:
            return [line2]
        if Ze.imag < -DELTA and not is_top:
            return [line3]



# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize


# TODO define one loss calculation function to use everywhere in class
def _comp_loss(self, C, Cx, f, B):
    f_norm = f / self.F_REF
    B_norm = B / self.B_REF

    ii = 0
    _C = list(C)
    for idx, c in enumerate(_C):
        if c is None:
            _C[idx] = Cx[ii]
            ii += 1

    loss = (
        _C[0] * f_norm * B_norm ** _C[1]
        + _C[2] * (f_norm * B_norm) ** _C[3]
        + _C[4] * (f_norm * B_norm) ** _C[5]
    )

    return loss


def comp_coeff_Bertotti(self, mat):
    """
    Compute the missing (i.e. None-valued) Bertotti loss coefficients from the
    Material object data by data fitting.

    Parameters
    ----------
    self : LossModelBertotti
        A LossModelBertotti object

    mat : Material
        A Material object

    Return
    ------
    success : bool
        Return 'True' if parameter fitting was successful.

    """
    # Get logger
    logger = self.get_logger()

    # store loss model parameter
    C = []
    C.append(self.k_hy)
    C.append(self.alpha_hy)
    C.append(self.k_ed)
    C.append(self.alpha_ed)
    C.append(self.k_ex)
    C.append(self.alpha_ex)

    # check if parameters have to been estimated
    n_est = sum(x is None for x in C)
    if n_est == 0:
        return True

    data = mat.mag.LossData.get_data()
    f = data[:, 1]
    B = data[:, 0]
    Loss = data[:, 2]

    # fit the data
    # TODO Which normalization to use? 1/f or should it be user defined?
    # TODO use constrained parameter estimation
    _comp_err = lambda Cx: (_comp_loss(self, C, Cx, f, B) - Loss) / (f)
    C0 = np.ones([n_est])  # initial values for the parameters
    result = optimize.least_squares(_comp_err, C0[:], method="lm")

    C1 = result.x
    success = result.success

    if success is None:
        logger.warning(f"'{self.name}' LossModel: Parameter fitting failed.")
        return False
    else:
        ii = 0
        for idx, c in enumerate(C):
            if c is None:
                C[idx] = C1[ii]
                ii += 1

        # set the estimated fitting parameters
        self.k_hy = C[0]
        self.alpha_hy = C[1]
        self.k_ed = C[2]
        self.alpha_ed = C[3]
        self.k_ex = C[4]
        self.alpha_ex = C[5]

        logger.debug(f"'{self.name}' LossModel: Parameters estimated coefficiants:")
        logger.debug(C)

        return True



# -*- coding: utf-8 -*-


def comp_surface_magnet_id(self, index):
    """Compute the surface of the hole magnet of the corresponding index

    Parameters
    ----------
    self : HoleM50
        A HoleM50 object
    index : int
        Index of the magnet to compute the surface

    Returns
    -------
    Smag: float
        Surface of the Magnet [m**2]
    """

    # all magnet has the same surface
    label = "magnet_" + str(index)
    if index in [0, 1, 2] and getattr(self, label) is not None:
        return self.H3 * self.W4
    return 0



# -*- coding: utf-8 -*-

from numpy import exp, sin

from ....Functions.Geometry.inter_line_circle import inter_line_circle
from ....Methods.Slot.HoleM53 import Slot53InterError


def comp_W5(self):
    """Compute the W5 width of the hole (cf schematics)

    Parameters
    ----------
    self : HoleM53
        a HoleM53 object

    Returns
    -------
    W5: float
        Cf schematics [m]
    Raises
    --------
    Slot53InterError
        ERROR: Slot 53, Can't find Z11 coordinates

    """

    Rext = self.get_Rext()

    Z7 = Rext - self.H0 - 1j * self.W1 / 2
    Z8 = Z7 + (self.H2 - self.H3) * sin(self.W4)
    Z10 = (self.W2 + self.W3) * exp(-1j * self.W4) + Z8

    # Z1 and Z11 are defined as intersection between line and circle
    Zlist = inter_line_circle(Z8, Z10, Rext - self.H1)
    if len(Zlist) == 2 and Zlist[0].imag < 0 and Zlist[0].real > 0:
        Z11 = Zlist[0]
    elif len(Zlist) == 2 and Zlist[1].imag < 0 and Zlist[1].real > 0:
        Z11 = Zlist[1]
    else:
        raise Slot53InterError("ERROR: Slot 53, Can't find Z11 coordinates")

    return abs(Z11 - Z8) - self.W2 - self.W3



# -*- coding: utf-8 -*-


def get_begin(self):
    """Return the begin point of the segment

    Parameters
    ----------
    self : Segment
        A Segment object

    Returns
    -------
    begin: complex
        Begin point of the Segment
    """

    return self.begin
